@@ -45,16 +45,14 @@ static BOOL init(void)
 {
     hadvapi32 = GetModuleHandle("advapi32.dll");
 
-    if (hadvapi32) {
-        pLsaClose = (void*)GetProcAddress(hadvapi32, "LsaClose");
-        pLsaFreeMemory = (void*)GetProcAddress(hadvapi32, "LsaFreeMemory");
-        pLsaOpenPolicy = (void*)GetProcAddress(hadvapi32, "LsaOpenPolicy");
-        pLsaQueryInformationPolicy = (void*)GetProcAddress(hadvapi32, "LsaQueryInformationPolicy");
-        pConvertSidToStringSidA = (void*)GetProcAddress(hadvapi32, "ConvertSidToStringSidA");
+    pLsaClose = (void*)GetProcAddress(hadvapi32, "LsaClose");
+    pLsaFreeMemory = (void*)GetProcAddress(hadvapi32, "LsaFreeMemory");
+    pLsaOpenPolicy = (void*)GetProcAddress(hadvapi32, "LsaOpenPolicy");
+    pLsaQueryInformationPolicy = (void*)GetProcAddress(hadvapi32, "LsaQueryInformationPolicy");
+    pConvertSidToStringSidA = (void*)GetProcAddress(hadvapi32, "ConvertSidToStringSidA");
 
-        if (pLsaClose && pLsaFreeMemory && pLsaOpenPolicy && pLsaQueryInformationPolicy && pConvertSidToStringSidA)
-            return TRUE;
-    }
+    if (pLsaClose && pLsaFreeMemory && pLsaOpenPolicy && pLsaQueryInformationPolicy && pConvertSidToStringSidA)
+        return TRUE;
 
     return FALSE;
 }
@@ -86,7 +84,10 @@ static void test_lsa(void)
         PPOLICY_DNS_DOMAIN_INFO dns_domain_info;
 
         status = pLsaQueryInformationPolicy(handle, PolicyAuditEventsInformation, (PVOID*)&audit_events_info);
-        ok(status == STATUS_SUCCESS, "LsaQueryInformationPolicy(PolicyAuditEventsInformation) failed, returned 0x%08x\n", status);
+        if (status == STATUS_ACCESS_DENIED)
+            skip("Not enough rights to retrieve PolicyAuditEventsInformation\n");
+        else
+            ok(status == STATUS_SUCCESS, "LsaQueryInformationPolicy(PolicyAuditEventsInformation) failed, returned 0x%08x\n", status);
         if (status == STATUS_SUCCESS) {
             pLsaFreeMemory((LPVOID)audit_events_info);
         }
