@@ -223,7 +223,7 @@ BOOL WINAPI ChooseFontW(LPCHOOSEFONTW lpChFont)
 /***********************************************************************
  *           ChooseFontA   (COMDLG32.@)
  *
- * See ChosseFontW.
+ * See ChooseFontW.
  */
 BOOL WINAPI ChooseFontA(LPCHOOSEFONTA lpChFont)
 {
@@ -357,24 +357,20 @@ static int SetFontStylesToCombo2(HWND hwnd, HDC hdc, const LOGFONTW *lplf)
     {
         int italic;
         int weight;
-        const WCHAR *stname;
+        UINT resId;
     };
-    static const WCHAR strRegular[]    = {'R','e','g','u','l','a','r',0};
-    static const WCHAR strItalic[]     = {'I','t','a','l','i','c',0};
-    static const WCHAR strBold[]       = {'B','o','l','d',0};
-    static const WCHAR strBoldItalic[] = {'B','o','l','d',' ','I','t','a','l','i','c',0};
     static const struct FONTSTYLE fontstyles[FSTYLES]={
-        { 0, FW_NORMAL, strRegular },
-        { 1, FW_NORMAL, strItalic },
-        { 0, FW_BOLD,   strBold },
-        { 1, FW_BOLD,   strBoldItalic }
+        { 0, FW_NORMAL, IDS_FONT_REGULAR },
+        { 1, FW_NORMAL, IDS_FONT_ITALIC },
+        { 0, FW_BOLD,   IDS_FONT_BOLD },
+        { 1, FW_BOLD,   IDS_FONT_BOLD_ITALIC }
     };
     HFONT hf;
     TEXTMETRICW tm;
     int i,j;
     LOGFONTW lf;
 
-    memcpy(&lf, lplf, sizeof(LOGFONTW));
+    lf = *lplf;
 
     for (i=0;i<FSTYLES;i++)
     {
@@ -390,7 +386,9 @@ static int SetFontStylesToCombo2(HWND hwnd, HDC hdc, const LOGFONTW *lplf)
              (fontstyles[i].weight == FW_BOLD && tm.tmWeight > FW_MEDIUM)) &&
             ((tm.tmItalic != 0)==fontstyles[i].italic))
         {
-            j=SendMessageW(hwnd,CB_ADDSTRING,0,(LPARAM)fontstyles[i].stname );
+            WCHAR name[64];
+            LoadStringW(COMDLG32_hInstance, fontstyles[i].resId, name, 64);
+            j=SendMessageW(hwnd,CB_ADDSTRING,0,(LPARAM)name );
             if (j==CB_ERR) return 1;
             j=SendMessageW(hwnd, CB_SETITEMDATA, j,
                            MAKELONG(tm.tmWeight,fontstyles[i].italic));
@@ -963,7 +961,7 @@ LRESULT CFn_WMCommand(HWND hDlg, WPARAM wParam, LPARAM lParam,
                    call back with the extra FONTTYPE_...  bits added */
                 lpxx->lfPitchAndFamily = HIWORD(l) >> 8;
             }
-            lstrcpyW(lpxx->lfFaceName,str);
+            lstrcpynW(lpxx->lfFaceName, str, sizeof(lpxx->lfFaceName)/sizeof(lpxx->lfFaceName[0]));
             i=SendDlgItemMessageW(hDlg, cmb2, CB_GETCURSEL, 0, 0);
             if (i!=CB_ERR)
             {
