@@ -310,7 +310,7 @@ IDirect3DImpl_3_EnumDevices(IDirect3D3 *iface,
     /* Some games (Motoracer 2 demo) have the bad idea to modify the device name string.
        Let's put the string in a sufficiently sized array in writable memory. */
     char device_name[50];
-    strcpy(device_name,"direct3d");
+    strcpy(device_name,"Direct3D HEL");
 
     TRACE("(%p)->(%p,%p)\n", This, Callback, Context);
     EnterCriticalSection(&ddraw_cs);
@@ -359,6 +359,8 @@ IDirect3DImpl_3_EnumDevices(IDirect3D3 *iface,
         }
     }
 
+    strcpy(device_name,"Direct3D HAL");
+
     TRACE("(%p) Enumerating HAL Direct3D device\n", This);
     d1 = dref;
     d2 = dref;
@@ -818,11 +820,13 @@ IDirect3DImpl_7_CreateDevice(IDirect3D7 *iface,
     object->Handles = NULL;
     object->numHandles = 0;
 
+    object->legacyTextureBlending = FALSE;
+
     /* This is for convenience */
     object->wineD3DDevice = This->wineD3DDevice;
 
     /* Create an index buffer, it's needed for indexed drawing */
-    IndexBufferParent = HeapAlloc(GetProcessHeap(), 0, sizeof(IParentImpl *));
+    IndexBufferParent = HeapAlloc(GetProcessHeap(), 0, sizeof(IParentImpl));
     if(!IndexBufferParent)
     {
         ERR("Allocating memory for an index buffer parent failed\n");
