@@ -64,7 +64,9 @@ static void KeyboardCallback( LPDIRECTINPUTDEVICE8A iface, WPARAM wparam, LPARAM
     TRACE("(%p) %ld,%ld\n", iface, wparam, lparam);
 
     dik_code = hook->scanCode & 0xff;
-    if (hook->flags & LLKHF_EXTENDED) dik_code |= 0x80;
+    /* R-Shift is special - it is an extended key with separate scan code */
+    if (hook->flags & LLKHF_EXTENDED && dik_code != 0x36)
+        dik_code |= 0x80;
 
     new_diks = hook->flags & LLKHF_UP ? 0 : 0x80;
 
@@ -268,7 +270,7 @@ static HRESULT WINAPI SysKeyboardAImpl_GetDeviceState(
 
     if (!This->base.acquired) return DIERR_NOTACQUIRED;
 
-    if (len != WINE_DINPUT_KEYBOARD_MAX_KEYS)
+    if (len != This->base.data_format.user_df->dwDataSize )
         return DIERR_INVALIDPARAM;
 
     EnterCriticalSection(&This->base.crit);
@@ -280,8 +282,8 @@ static HRESULT WINAPI SysKeyboardAImpl_GetDeviceState(
 		TRACE(" - %02X: %02x\n", i, This->DInputKeyState[i]);
 	}
     }
-    
-    memcpy(ptr, This->DInputKeyState, WINE_DINPUT_KEYBOARD_MAX_KEYS);
+
+    fill_DataFormat(ptr, This->DInputKeyState, &This->base.data_format);
     LeaveCriticalSection(&This->base.crit);
 
     return DI_OK;
