@@ -134,7 +134,7 @@ static HRESULT WINAPI IDirectMusic8Impl_CreatePort (LPDIRECTMUSIC8 iface, REFCLS
 			if (!This->ppPorts) This->ppPorts = HeapAlloc(GetProcessHeap(), 0, sizeof(LPDIRECTMUSICPORT) * This->nrofports);
 			else This->ppPorts = HeapReAlloc(GetProcessHeap(), 0, This->ppPorts, sizeof(LPDIRECTMUSICPORT) * This->nrofports); 			
 			This->ppPorts[This->nrofports - 1] = pNewPort;
-			*ppPort = (LPDIRECTMUSICPORT) pNewPort;
+			*ppPort = pNewPort;
 			return S_OK;			
 		}
 	}
