@@ -31,4 +31,4 @@
 #include "initguid.h"
 
 #include "d3d8.h"
-#include "d3dx8core.h"
+#include "d3dx8.h"
