@@ -28,7 +28,6 @@
 #include "windef.h"
 #include "winbase.h"
 #include "wingdi.h"
-#include "wine/wingdi16.h"
 #include "gdi_private.h"
 #include "wine/debug.h"
 
@@ -43,13 +42,11 @@ typedef struct
 
 
 static HGDIOBJ PEN_SelectObject( HGDIOBJ handle, HDC hdc );
-static INT PEN_GetObject16( HGDIOBJ handle, void *obj, INT count, LPVOID buffer );
 static INT PEN_GetObject( HGDIOBJ handle, void *obj, INT count, LPVOID buffer );
 
 static const struct gdi_obj_funcs pen_funcs =
 {
     PEN_SelectObject,  /* pSelectObject */
-    PEN_GetObject16,   /* pGetObject16 */
     PEN_GetObject,     /* pGetObjectA */
     PEN_GetObject,     /* pGetObjectW */
     NULL,              /* pUnrealizeObject */
@@ -185,7 +182,7 @@ HPEN WINAPI ExtCreatePen( DWORD style, DWORD width,
     }
     else
     {
-        /* PS_INSIDEFRAME is applicable only for gemetric pens */
+        /* PS_INSIDEFRAME is applicable only for geometric pens */
         if ((style & PS_STYLE_MASK) == PS_INSIDEFRAME || width != 1)
         {
             SetLastError(ERROR_INVALID_PARAMETER);
@@ -218,45 +215,35 @@ HPEN WINAPI ExtCreatePen( DWORD style, DWORD width,
 static HGDIOBJ PEN_SelectObject( HGDIOBJ handle, HDC hdc )
 {
     HGDIOBJ ret = 0;
-    DC *dc = DC_GetDCPtr( hdc );
+    DC *dc = get_dc_ptr( hdc );
 
-    if (!dc) return 0;
+    if (!dc)
+    {
+        SetLastError( ERROR_INVALID_HANDLE );
+        return 0;
+    }
 
-    if (dc->funcs->pSelectPen) handle = dc->funcs->pSelectPen( dc->physDev, handle );
-    if (handle)
+    if (!GDI_inc_ref_count( handle ))
+    {
+        release_dc_ptr( dc );
+        return 0;
+    }
+
+    if (dc->funcs->pSelectPen && !dc->funcs->pSelectPen( dc->physDev, handle ))
+    {
+        GDI_dec_ref_count( handle );
+    }
+    else
     {
         ret = dc->hPen;
         dc->hPen = handle;
-        GDI_inc_ref_count( handle );
         GDI_dec_ref_count( ret );
     }
-    DC_ReleaseDCPtr( dc );
+    release_dc_ptr( dc );
     return ret;
 }
 
 
-/***********************************************************************
- *           PEN_GetObject16
- */
-static INT PEN_GetObject16( HGDIOBJ handle, void *obj, INT count, LPVOID buffer )
-{
-    PENOBJ *pen = obj;
-    LOGPEN16 *logpen;
-
-    if (!buffer) return sizeof(LOGPEN16);
-
-    if (count < sizeof(LOGPEN16)) return 0;
-
-    logpen = buffer;
-    logpen->lopnStyle = pen->logpen.elpPenStyle;
-    logpen->lopnColor = pen->logpen.elpColor;
-    logpen->lopnWidth.x = pen->logpen.elpWidth;
-    logpen->lopnWidth.y = 0;
-
-    return sizeof(LOGPEN16);
-}
-
-
 /***********************************************************************
  *           PEN_GetObject
  */
@@ -278,7 +265,7 @@ static INT PEN_GetObject( HGDIOBJ handle, void *obj, INT count, LPVOID buffer )
             count == sizeof(EXTLOGPEN))
         {
             EXTLOGPEN *elp = buffer;
-            memcpy(elp, &pen->logpen, sizeof(EXTLOGPEN));
+            *elp = pen->logpen;
             elp->elpWidth = 0;
             return sizeof(EXTLOGPEN);
         }
