@@ -19,6 +19,8 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
+#define WINVER 0x0501 /* request latest DEVMODE */
+
 #include <assert.h>
 #include <stdio.h>
 
@@ -173,8 +175,88 @@ static void test_savedc(void)
     DeleteDC(hdc);
 }
 
+static void test_GdiConvertToDevmodeW(void)
+{
+    DEVMODEW * (WINAPI *pGdiConvertToDevmodeW)(const DEVMODEA *);
+    DEVMODEA dmA;
+    DEVMODEW *dmW;
+    BOOL ret;
+
+    pGdiConvertToDevmodeW = (void *)GetProcAddress(GetModuleHandleA("gdi32.dll"), "GdiConvertToDevmodeW");
+    if (!pGdiConvertToDevmodeW)
+    {
+        skip("GdiConvertToDevmodeW is not available on this platform\n");
+        return;
+    }
+
+    ret = EnumDisplaySettingsA(NULL, ENUM_CURRENT_SETTINGS, &dmA);
+    ok(ret, "EnumDisplaySettingsExA error %u\n", GetLastError());
+    ok(dmA.dmSize >= FIELD_OFFSET(DEVMODEA, dmICMMethod), "dmSize is too small: %04x\n", dmA.dmSize);
+    ok(dmA.dmSize <= sizeof(DEVMODEA), "dmSize is too large: %04x\n", dmA.dmSize);
+
+    dmW = pGdiConvertToDevmodeW(&dmA);
+    ok(dmW->dmSize >= FIELD_OFFSET(DEVMODEW, dmICMMethod), "dmSize is too small: %04x\n", dmW->dmSize);
+    ok(dmW->dmSize <= sizeof(DEVMODEW), "dmSize is too large: %04x\n", dmW->dmSize);
+    HeapFree(GetProcessHeap(), 0, dmW);
+
+    dmA.dmSize = FIELD_OFFSET(DEVMODEA, dmFields) + sizeof(dmA.dmFields);
+    dmW = pGdiConvertToDevmodeW(&dmA);
+    ok(dmW->dmSize == FIELD_OFFSET(DEVMODEW, dmFields) + sizeof(dmW->dmFields),
+       "wrong size %u\n", dmW->dmSize);
+    HeapFree(GetProcessHeap(), 0, dmW);
+
+    dmA.dmICMMethod = DMICMMETHOD_NONE;
+    dmA.dmSize = FIELD_OFFSET(DEVMODEA, dmICMMethod) + sizeof(dmA.dmICMMethod);
+    dmW = pGdiConvertToDevmodeW(&dmA);
+    ok(dmW->dmSize == FIELD_OFFSET(DEVMODEW, dmICMMethod) + sizeof(dmW->dmICMMethod),
+       "wrong size %u\n", dmW->dmSize);
+    ok(dmW->dmICMMethod == DMICMMETHOD_NONE,
+       "expected DMICMMETHOD_NONE, got %u\n", dmW->dmICMMethod);
+    HeapFree(GetProcessHeap(), 0, dmW);
+
+    dmA.dmSize = 1024;
+    dmW = pGdiConvertToDevmodeW(&dmA);
+    ok(dmW->dmSize == FIELD_OFFSET(DEVMODEW, dmPanningHeight) + sizeof(dmW->dmPanningHeight),
+       "wrong size %u\n", dmW->dmSize);
+    HeapFree(GetProcessHeap(), 0, dmW);
+
+    SetLastError(0xdeadbeef);
+    dmA.dmSize = 0;
+    dmW = pGdiConvertToDevmodeW(&dmA);
+    ok(!dmW, "GdiConvertToDevmodeW should fail\n");
+    ok(GetLastError() == 0xdeadbeef, "expected 0xdeadbeef, got %u\n", GetLastError());
+
+    /* this is the minimal dmSize that XP accepts */
+    dmA.dmSize = FIELD_OFFSET(DEVMODEA, dmFields);
+    dmW = pGdiConvertToDevmodeW(&dmA);
+    ok(dmW->dmSize == FIELD_OFFSET(DEVMODEW, dmFields),
+       "expected %04x, got %04x\n", FIELD_OFFSET(DEVMODEW, dmFields), dmW->dmSize);
+    HeapFree(GetProcessHeap(), 0, dmW);
+}
+
+static void test_CreateCompatibleDC(void)
+{
+    BOOL bRet;
+    HDC hDC;
+    HDC hNewDC;
+
+    /* Create a DC compatible with the screen */
+    hDC = CreateCompatibleDC(NULL);
+    ok(hDC != NULL, "CreateCompatibleDC returned %p\n", hDC);
+
+    /* Delete this DC, this should succeed */
+    bRet = DeleteDC(hDC);
+    ok(bRet == TRUE, "DeleteDC returned %u\n", bRet);
+
+    /* Try to create a DC compatible to the deleted DC. This has to fail */
+    hNewDC = CreateCompatibleDC(hDC);
+    ok(hNewDC == NULL, "CreateCompatibleDC returned %p\n", hNewDC);
+}
+
 START_TEST(dc)
 {
     test_savedc();
     test_savedc_2();
+    test_GdiConvertToDevmodeW();
+    test_CreateCompatibleDC();
 }
