@@ -72,7 +72,7 @@
 #include "icmpapi.h"
 #include "wine/debug.h"
 
-/* Set up endiannes macros for the ip and ip_icmp BSD headers */
+/* Set up endianness macros for the ip and ip_icmp BSD headers */
 #ifndef BIG_ENDIAN
 #define BIG_ENDIAN       4321
 #endif
@@ -433,8 +433,9 @@ DWORD WINAPI IcmpSendEcho(
              * Decrease the timeout so that we don't enter an endless loop even
              * if we get flooded with ICMP packets that are not for us.
              */
-            Timeout -= (recv_time - send_time);
-            if (Timeout < 0) Timeout = 0;
+            DWORD t = (recv_time - send_time);
+            if (Timeout > t) Timeout -= t;
+            else             Timeout = 0;
             continue;
         } else {
             /* This is a reply to our packet */
