@@ -73,7 +73,7 @@ typedef	struct
     int		rate;
 } Format;
 
-static Format PCM_Formats[] =
+static const Format PCM_Formats[] =
 {
     {1,  8,  8000}, {2,  8,  8000}, {1, 16,  8000}, {2, 16,  8000},
     {1,  8, 11025}, {2,  8, 11025}, {1, 16, 11025}, {2, 16, 11025},
@@ -81,7 +81,7 @@ static Format PCM_Formats[] =
     {1,  8, 44100}, {2,  8, 44100}, {1, 16, 44100}, {2, 16, 44100},
 };
 
-static Format ADPCM_Formats[] =
+static const Format ADPCM_Formats[] =
 {
     {1,  4,  8000}, {2,	4,  8000},  {1,  4, 11025}, {2,	 4, 11025},
     {1,  4, 22050}, {2,	4, 22050},  {1,  4, 44100}, {2,	 4, 44100},
@@ -93,10 +93,10 @@ static Format ADPCM_Formats[] =
 /***********************************************************************
  *           ADPCM_GetFormatIndex
  */
-static	DWORD	ADPCM_GetFormatIndex(LPWAVEFORMATEX wfx)
+static	DWORD	ADPCM_GetFormatIndex(const WAVEFORMATEX *wfx)
 {
-    int 	i, hi;
-    Format*	fmts;
+    int             i, hi;
+    const Format*   fmts;
 
     switch (wfx->wFormatTag)
     {
@@ -711,7 +711,7 @@ static	LRESULT	ADPCM_StreamOpen(PACMDRVSTREAMINSTANCE adsi)
 
         /* we check that in a block, after the header, samples are present on
          * 4-sample packet pattern
-         * we also check that the block alignement is bigger than the expected size
+         * we also check that the block alignment is bigger than the expected size
          */
         if (((nspb - 1) & 3) != 0) goto theEnd;
         if ((((nspb - 1) / 2) + 4) * adsi->pwfxSrc->nChannels < adsi->pwfxSrc->nBlockAlign)
@@ -736,7 +736,7 @@ static	LRESULT	ADPCM_StreamOpen(PACMDRVSTREAMINSTANCE adsi)
 
         /* we check that in a block, after the header, samples are present on
          * 4-sample packet pattern
-         * we also check that the block alignement is bigger than the expected size
+         * we also check that the block alignment is bigger than the expected size
          */
         if (((nspb - 1) & 3) != 0) goto theEnd;
         if ((((nspb - 1) / 2) + 4) * adsi->pwfxDst->nChannels < adsi->pwfxDst->nBlockAlign)
@@ -773,7 +773,7 @@ static	LRESULT	ADPCM_StreamClose(PACMDRVSTREAMINSTANCE adsi)
  *           ADPCM_StreamSize
  *
  */
-static	LRESULT ADPCM_StreamSize(PACMDRVSTREAMINSTANCE adsi, PACMDRVSTREAMSIZE adss)
+static	LRESULT ADPCM_StreamSize(const ACMDRVSTREAMINSTANCE *adsi, PACMDRVSTREAMSIZE adss)
 {
     switch (adss->fdwSize)
     {
