@@ -738,6 +738,7 @@ DWORD WINAPI GetAdaptersInfo(PIP_ADAPTER_INFO pAdapterInfo, PULONG pOutBufLen)
 
               /* on Win98 this is left empty, but whatever */
               getInterfaceNameByIndex(table->indexes[ndx], ptr->AdapterName);
+              getInterfaceNameByIndex(table->indexes[ndx], ptr->Description);
               ptr->AddressLength = sizeof(ptr->Address);
               getInterfacePhysicalByIndex(table->indexes[ndx],
                &ptr->AddressLength, ptr->Address, &ptr->Type);
@@ -1797,7 +1798,7 @@ DWORD WINAPI IpReleaseAddress(PIP_ADAPTER_INDEX_MAP AdapterInfo)
 /******************************************************************
  *    IpRenewAddress (IPHLPAPI.@)
  *
- * Renew an IP optained through DHCP.
+ * Renew an IP obtained through DHCP.
  *
  * PARAMS
  *  AdapterInfo [In] adapter to renew IP address
@@ -1851,7 +1852,7 @@ DWORD WINAPI NotifyAddrChange(PHANDLE Handle, LPOVERLAPPED overlapped)
  * Notify caller whenever the ip routing table is changed.
  *
  * PARAMS
- *  Handle     [Out] handle useable in asynchronus notification
+ *  Handle     [Out] handle usable in asynchronous notification
  *  overlapped [In]  overlapped structure that notifies the caller
  *
  * RETURNS
