@@ -1446,7 +1446,13 @@ static void test_FindFirstFileExA(void)
     _lclose(_lcreat("test-dir\\file2", 0));
     CreateDirectoryA("test-dir\\dir1", NULL);
     /* FindExLimitToDirectories is ignored */
+    SetLastError(0xdeadbeef);
     handle = pFindFirstFileExA("test-dir\\*", FindExInfoStandard, &search_results, FindExSearchLimitToDirectories, NULL, 0);
+    if (handle == INVALID_HANDLE_VALUE && GetLastError() == ERROR_CALL_NOT_IMPLEMENTED)
+    {
+        skip("FindFirstFileExA is not implemented\n");
+        goto cleanup;
+    }
     ok(handle != INVALID_HANDLE_VALUE, "FindFirstFile failed (err=%u)\n", GetLastError());
     ok(strcmp(search_results.cFileName, ".") == 0, "First entry should be '.', is %s\n", search_results.cFileName);
 
@@ -1456,7 +1462,7 @@ static void test_FindFirstFileExA(void)
     ok(strcmp(search_results.cFileName, "..") == 0, "Second entry should be '..' is %s\n", search_results.cFileName);
 
     ok(FindNextFile(handle, &search_results), "Fetching third file failed\n");
-    ok(CHECK_NAME(search_results.cFileName), "Invalid thrid entry - %s\n", search_results.cFileName);
+    ok(CHECK_NAME(search_results.cFileName), "Invalid third entry - %s\n", search_results.cFileName);
 
     ok(FindNextFile(handle, &search_results), "Fetching fourth file failed\n");
     ok(CHECK_NAME(search_results.cFileName), "Invalid fourth entry - %s\n", search_results.cFileName);
@@ -1467,6 +1473,8 @@ static void test_FindFirstFileExA(void)
 #undef CHECK_NAME
 
     ok(FindNextFile(handle, &search_results) == FALSE, "Fetching sixth file should failed\n");
+
+cleanup:
     DeleteFileA("test-dir\\file1");
     DeleteFileA("test-dir\\file2");
     RemoveDirectoryA("test-dir\\dir1");
@@ -1840,14 +1848,14 @@ static void test_overlapped(void)
     memset( &ov, 0,  sizeof ov );
     result = 1;
     r = GetOverlappedResult(0, &ov, &result, 0);
-    ok( r == TRUE, "should return false\n");
+    ok( r == TRUE, "should return true\n");
     ok( result == 0, "wrong result %u\n", result );
 
     result = 0;
     ov.Internal = 0;
     ov.InternalHigh = 0xabcd;
     r = GetOverlappedResult(0, &ov, &result, 0);
-    ok( r == TRUE, "should return false\n");
+    ok( r == TRUE, "should return true\n");
     ok( result == 0xabcd, "wrong result %u\n", result );
 
     SetLastError( 0xb00 );
