@@ -43,18 +43,80 @@ static void init_function_pointers(void)
     KERNEL32_GET_PROC(VerSetConditionMask);
 }
 
-START_TEST(version)
+static void test_GetVersionEx(void)
+{
+    OSVERSIONINFOA infoA;
+    OSVERSIONINFOEXA infoExA;
+    BOOL ret;
+
+    if (0)
+    {
+        /* Silently crashes on XP */
+        ret = GetVersionExA(NULL);
+    }
+
+    SetLastError(0xdeadbeef);
+    memset(&infoA,0,sizeof infoA);
+    ret = GetVersionExA(&infoA);
+    ok(!ret, "Expected GetVersionExA to fail\n");
+    ok(GetLastError() == ERROR_INSUFFICIENT_BUFFER ||
+        GetLastError() == 0xdeadbeef /* Win9x */,
+        "Expected ERROR_INSUFFICIENT_BUFFER or 0xdeadbeef (Win9x), got %d\n",
+        GetLastError());
+
+    SetLastError(0xdeadbeef);
+    infoA.dwOSVersionInfoSize = sizeof(OSVERSIONINFOA) / 2;
+    ret = GetVersionExA(&infoA);
+    ok(!ret, "Expected GetVersionExA to fail\n");
+    ok(GetLastError() == ERROR_INSUFFICIENT_BUFFER ||
+        GetLastError() == 0xdeadbeef /* Win9x */,
+        "Expected ERROR_INSUFFICIENT_BUFFER or 0xdeadbeef (Win9x), got %d\n",
+        GetLastError());
+
+    SetLastError(0xdeadbeef);
+    infoA.dwOSVersionInfoSize = sizeof(OSVERSIONINFOA) * 2;
+    ret = GetVersionExA(&infoA);
+    ok(!ret, "Expected GetVersionExA to fail\n");
+    ok(GetLastError() == ERROR_INSUFFICIENT_BUFFER ||
+        GetLastError() == 0xdeadbeef /* Win9x */,
+        "Expected ERROR_INSUFFICIENT_BUFFER or 0xdeadbeef (Win9x), got %d\n",
+        GetLastError());
+
+    SetLastError(0xdeadbeef);
+    infoA.dwOSVersionInfoSize = sizeof(OSVERSIONINFOA);
+    ret = GetVersionExA(&infoA);
+    ok(ret, "Expected GetVersionExA to succeed\n");
+    ok(GetLastError() == 0xdeadbeef,
+        "Expected 0xdeadbeef, got %d\n", GetLastError());
+
+    SetLastError(0xdeadbeef);
+    infoExA.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEXA);
+    ret = GetVersionExA((OSVERSIONINFOA *)&infoExA);
+    ok(ret, "Expected GetVersionExA to succeed\n");
+    ok(GetLastError() == 0xdeadbeef,
+        "Expected 0xdeadbeef, got %d\n", GetLastError());
+}
+
+static void test_VerifyVersionInfo(void)
 {
     OSVERSIONINFOEX info = { sizeof(info) };
     BOOL ret;
+    DWORD servicepack;
 
-    init_function_pointers();
     if(!pVerifyVersionInfoA || !pVerSetConditionMask)
     {
         skip("Needed functions not available\n");
         return;
     }
 
+    /* Before we start doing some tests we should check what the version of
+     * the ServicePack is. Tests on a box with no ServicePack will fail otherwise.
+     */
+    GetVersionEx((OSVERSIONINFO *)&info);
+    servicepack = info.wServicePackMajor;
+    memset(&info, 0, sizeof(info));
+    info.dwOSVersionInfoSize = sizeof(info);
+
     ret = pVerifyVersionInfoA(&info, VER_MAJORVERSION | VER_MINORVERSION,
         pVerSetConditionMask(0, VER_MAJORVERSION, VER_GREATER_EQUAL));
     ok(ret, "VerifyVersionInfoA failed with error %d\n", GetLastError());
@@ -99,7 +161,11 @@ START_TEST(version)
     ret = pVerifyVersionInfoA(&info, VER_MINORVERSION | VER_SERVICEPACKMAJOR | VER_SERVICEPACKMINOR,
         pVerSetConditionMask(pVerSetConditionMask(0, VER_MINORVERSION, VER_GREATER_EQUAL),
             VER_MAJORVERSION, VER_GREATER_EQUAL));
-    ok(ret, "VerifyVersionInfoA failed with error %d\n", GetLastError());
+    if (servicepack == 0)
+        ok(!ret && (GetLastError() == ERROR_OLD_WIN_VERSION),
+            "VerifyVersionInfoA should have failed with ERROR_OLD_WIN_VERSION instead of %d\n", GetLastError());
+    else
+        ok(ret, "VerifyVersionInfoA failed with error %d\n", GetLastError());
 
     GetVersionEx((OSVERSIONINFO *)&info);
     info.wServicePackMinor++;
@@ -108,17 +174,24 @@ START_TEST(version)
     ok(!ret && (GetLastError() == ERROR_OLD_WIN_VERSION),
         "VerifyVersionInfoA should have failed with ERROR_OLD_WIN_VERSION instead of %d\n", GetLastError());
 
-    GetVersionEx((OSVERSIONINFO *)&info);
-    info.wServicePackMajor--;
-    ret = pVerifyVersionInfoA(&info, VER_MINORVERSION | VER_SERVICEPACKMAJOR | VER_SERVICEPACKMINOR,
-        pVerSetConditionMask(0, VER_MINORVERSION, VER_GREATER));
-    ok(ret, "VerifyVersionInfoA failed with error %d\n", GetLastError());
+    if (servicepack == 0)
+    {
+        skip("There is no ServicePack on this system\n");
+    }
+    else
+    {
+        GetVersionEx((OSVERSIONINFO *)&info);
+        info.wServicePackMajor--;
+        ret = pVerifyVersionInfoA(&info, VER_MINORVERSION | VER_SERVICEPACKMAJOR | VER_SERVICEPACKMINOR,
+            pVerSetConditionMask(0, VER_MINORVERSION, VER_GREATER));
+        ok(ret, "VerifyVersionInfoA failed with error %d\n", GetLastError());
 
-    GetVersionEx((OSVERSIONINFO *)&info);
-    info.wServicePackMajor--;
-    ret = pVerifyVersionInfoA(&info, VER_MINORVERSION | VER_SERVICEPACKMAJOR | VER_SERVICEPACKMINOR,
-        pVerSetConditionMask(0, VER_MINORVERSION, VER_GREATER_EQUAL));
-    ok(ret, "VerifyVersionInfoA failed with error %d\n", GetLastError());
+        GetVersionEx((OSVERSIONINFO *)&info);
+        info.wServicePackMajor--;
+        ret = pVerifyVersionInfoA(&info, VER_MINORVERSION | VER_SERVICEPACKMAJOR | VER_SERVICEPACKMINOR,
+            pVerSetConditionMask(0, VER_MINORVERSION, VER_GREATER_EQUAL));
+        ok(ret, "VerifyVersionInfoA failed with error %d\n", GetLastError());
+    }
 
     GetVersionEx((OSVERSIONINFO *)&info);
     info.wServicePackMajor++;
@@ -185,3 +258,11 @@ START_TEST(version)
         pVerSetConditionMask(0, VER_MAJORVERSION, VER_GREATER_EQUAL));
     ok(ret, "VerifyVersionInfoA failed with error %d\n", GetLastError());
 }
+
+START_TEST(version)
+{
+    init_function_pointers();
+
+    test_GetVersionEx();
+    test_VerifyVersionInfo();
+}
