@@ -24,6 +24,7 @@
 #include "winbase.h"
 #include "winerror.h"
 #include "objbase.h"
+#include "initguid.h"
 #include "mapix.h"
 #include "mapiform.h"
 #include "mapi.h"
@@ -52,6 +53,16 @@ BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID fImpLoad)
     return TRUE;
 }
 
+/***********************************************************************
+ *		DllGetClassObject (MAPI32.27)
+ */
+HRESULT WINAPI DllGetClassObject(REFCLSID rclsid, REFIID iid, LPVOID *ppv)
+{
+    *ppv = NULL;
+    FIXME("\n\tCLSID:\t%s,\n\tIID:\t%s\n", debugstr_guid(rclsid), debugstr_guid(iid));
+    return CLASS_E_CLASSNOTAVAILABLE;
+}
+
 /***********************************************************************
  * DllCanUnloadNow (MAPI32.28)
  *
