@@ -16,8 +16,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "config.h"
-
 #include <stdarg.h>
 #include <stdio.h>
 
@@ -26,7 +24,6 @@
 #include "windef.h"
 #include "winbase.h"
 #include "winuser.h"
-#include "winnls.h"
 #include "ole2.h"
 #include "mshtmcid.h"
 
@@ -387,7 +384,7 @@ static void set_font_size(HTMLDocument *This, LPCWSTR size)
     if(collapsed) {
         nsISelection_Collapse(nsselection, (nsIDOMNode*)elem, 0);
     }else {
-        /* Remove all size attrbutes from the range */
+        /* Remove all size attributes from the range */
         remove_child_attr(elem, wszFont, &size_str);
         nsISelection_SelectAllChildren(nsselection, (nsIDOMNode*)elem);
     }
