@@ -29,6 +29,7 @@
 #include "msiquery.h"
 #include "objbase.h"
 #include "msipriv.h"
+#include "query.h"
 
 #include "wine/debug.h"
 
@@ -39,7 +40,7 @@ WINE_DEFAULT_DEBUG_CHANNEL(msidb);
 
 typedef struct tabSTREAM
 {
-    int str_index;
+    UINT str_index;
     LPWSTR name;
     IStream *stream;
 } STREAM;
@@ -54,9 +55,9 @@ typedef struct tagMSISTREAMSVIEW
     UINT row_size;
 } MSISTREAMSVIEW;
 
-static BOOL add_stream_to_table(MSISTREAMSVIEW *sv, STREAM *stream, int index)
+static BOOL streams_set_table_size(MSISTREAMSVIEW *sv, UINT size)
 {
-    if (index >= sv->max_streams)
+    if (size >= sv->max_streams)
     {
         sv->max_streams *= 2;
         sv->streams = msi_realloc(sv->streams, sv->max_streams * sizeof(STREAM *));
@@ -64,7 +65,6 @@ static BOOL add_stream_to_table(MSISTREAMSVIEW *sv, STREAM *stream, int index)
             return FALSE;
     }
 
-    sv->streams[index] = stream;
     return TRUE;
 }
 
@@ -139,12 +139,6 @@ static UINT STREAMS_get_row( struct tagMSIVIEW *view, UINT row, MSIRECORD **rec
 }
 
 static UINT STREAMS_set_row(struct tagMSIVIEW *view, UINT row, MSIRECORD *rec, UINT mask)
-{
-    FIXME("(%p, %d, %p, %d): stub!\n", view, row, rec, mask);
-    return ERROR_SUCCESS;
-}
-
-static UINT STREAMS_insert_row(struct tagMSIVIEW *view, MSIRECORD *rec, BOOL temporary)
 {
     MSISTREAMSVIEW *sv = (MSISTREAMSVIEW *)view;
     STREAM *stream;
@@ -156,7 +150,10 @@ static UINT STREAMS_insert_row(struct tagMSIVIEW *view, MSIRECORD *rec, BOOL tem
     ULONG count;
     UINT r = ERROR_FUNCTION_FAILED;
 
-    TRACE("(%p, %p, %d)\n", view, rec, temporary);
+    TRACE("(%p, %p)\n", view, rec);
+
+    if (row > sv->num_rows)
+        return ERROR_FUNCTION_FAILED;
 
     r = MSI_RecordGetIStream(rec, 2, &stm);
     if (r != ERROR_SUCCESS)
@@ -201,8 +198,7 @@ static UINT STREAMS_insert_row(struct tagMSIVIEW *view, MSIRECORD *rec, BOOL tem
     IStorage_OpenStream(sv->db->storage, name, 0,
                         STGM_READ | STGM_SHARE_EXCLUSIVE, 0, &stream->stream);
 
-    if (!add_stream_to_table(sv, stream, sv->num_rows++))
-        goto done;
+    sv->streams[row] = stream;
 
 done:
     msi_free(name);
@@ -213,6 +209,16 @@ done:
     return r;
 }
 
+static UINT STREAMS_insert_row(struct tagMSIVIEW *view, MSIRECORD *rec, BOOL temporary)
+{
+    MSISTREAMSVIEW *sv = (MSISTREAMSVIEW *)view;
+
+    if (!streams_set_table_size(sv, ++sv->num_rows))
+        return ERROR_FUNCTION_FAILED;
+
+    return STREAMS_set_row(view, sv->num_rows - 1, rec, 0);
+}
+
 static UINT STREAMS_delete_row(struct tagMSIVIEW *view, UINT row)
 {
     FIXME("(%p %d): stub!\n", view, row);
@@ -278,16 +284,96 @@ static UINT STREAMS_get_column_info(struct tagMSIVIEW *view,
     return ERROR_SUCCESS;
 }
 
+static UINT streams_find_row(MSISTREAMSVIEW *sv, MSIRECORD *rec, UINT *row)
+{
+    LPCWSTR str;
+    UINT i, id, data;
+
+    str = MSI_RecordGetString(rec, 1);
+    msi_string2idW(sv->db->strings, str, &id);
+
+    for (i = 0; i < sv->num_rows; i++)
+    {
+        STREAMS_fetch_int(&sv->view, i, 1, &data);
+
+        if (data == id)
+        {
+            *row = i;
+            return ERROR_SUCCESS;
+        }
+    }
+
+    return ERROR_FUNCTION_FAILED;
+}
+
+static UINT streams_modify_update(struct tagMSIVIEW *view, MSIRECORD *rec)
+{
+    MSISTREAMSVIEW *sv = (MSISTREAMSVIEW *)view;
+    UINT r, row;
+
+    r = streams_find_row(sv, rec, &row);
+    if (r != ERROR_SUCCESS)
+        return ERROR_FUNCTION_FAILED;
+
+    return STREAMS_set_row(view, row, rec, 0);
+}
+
+static UINT streams_modify_assign(struct tagMSIVIEW *view, MSIRECORD *rec)
+{
+    MSISTREAMSVIEW *sv = (MSISTREAMSVIEW *)view;
+    UINT r, row;
+
+    r = streams_find_row(sv, rec, &row);
+    if (r == ERROR_SUCCESS)
+        return streams_modify_update(view, rec);
+
+    return STREAMS_insert_row(view, rec, FALSE);
+}
+
 static UINT STREAMS_modify(struct tagMSIVIEW *view, MSIMODIFY eModifyMode, MSIRECORD *rec, UINT row)
 {
-    FIXME("(%p, %d, %p): stub!\n", view, eModifyMode, rec);
-    return ERROR_SUCCESS;
+    UINT r;
+
+    TRACE("%p %d %p\n", view, eModifyMode, rec);
+
+    switch (eModifyMode)
+    {
+    case MSIMODIFY_ASSIGN:
+        r = streams_modify_assign(view, rec);
+        break;
+
+    case MSIMODIFY_INSERT:
+        r = STREAMS_insert_row(view, rec, FALSE);
+        break;
+
+    case MSIMODIFY_UPDATE:
+        r = streams_modify_update(view, rec);
+        break;
+
+    case MSIMODIFY_VALIDATE_NEW:
+    case MSIMODIFY_INSERT_TEMPORARY:
+    case MSIMODIFY_REFRESH:
+    case MSIMODIFY_REPLACE:
+    case MSIMODIFY_MERGE:
+    case MSIMODIFY_DELETE:
+    case MSIMODIFY_VALIDATE:
+    case MSIMODIFY_VALIDATE_FIELD:
+    case MSIMODIFY_VALIDATE_DELETE:
+        FIXME("%p %d %p - mode not implemented\n", view, eModifyMode, rec );
+        r = ERROR_CALL_NOT_IMPLEMENTED;
+        break;
+
+    default:
+        r = ERROR_INVALID_DATA;
+    }
+
+    return r;
 }
 
 static UINT STREAMS_delete(struct tagMSIVIEW *view)
 {
     MSISTREAMSVIEW *sv = (MSISTREAMSVIEW *)view;
-    int i;
+    UINT i;
 
     TRACE("(%p)\n", view);
 
@@ -351,9 +437,10 @@ static const MSIVIEWOPS streams_ops =
     NULL,
     NULL,
     NULL,
+    NULL,
 };
 
-static UINT add_streams_to_table(MSISTREAMSVIEW *sv)
+static INT add_streams_to_table(MSISTREAMSVIEW *sv)
 {
     IEnumSTATSTG *stgenum = NULL;
     STATSTG stat;
@@ -379,23 +466,30 @@ static UINT add_streams_to_table(MSISTREAMSVIEW *sv)
 
         /* table streams are not in the _Streams table */
         if (*stat.pwcsName == 0x4840)
+        {
+            CoTaskMemFree(stat.pwcsName);
             continue;
+        }
 
         stream = create_stream(sv, stat.pwcsName, TRUE, NULL);
         if (!stream)
         {
             count = -1;
+            CoTaskMemFree(stat.pwcsName);
             break;
         }
 
         IStorage_OpenStream(sv->db->storage, stat.pwcsName, 0,
                             STGM_READ | STGM_SHARE_EXCLUSIVE, 0, &stream->stream);
+        CoTaskMemFree(stat.pwcsName);
 
-        if (!add_stream_to_table(sv, stream, count++))
+        if (!streams_set_table_size(sv, ++count))
         {
             count = -1;
             break;
         }
+
+        sv->streams[count - 1] = stream;
     }
 
     IEnumSTATSTG_Release(stgenum);
@@ -405,6 +499,7 @@ static UINT add_streams_to_table(MSISTREAMSVIEW *sv)
 UINT STREAMS_CreateView(MSIDATABASE *db, MSIVIEW **view)
 {
     MSISTREAMSVIEW *sv;
+    INT rows;
 
     TRACE("(%p, %p)\n", db, view);
 
@@ -414,10 +509,10 @@ UINT STREAMS_CreateView(MSIDATABASE *db, MSIVIEW **view)
 
     sv->view.ops = &streams_ops;
     sv->db = db;
-    sv->num_rows = add_streams_to_table(sv);
-
-    if (sv->num_rows < 0)
+    rows = add_streams_to_table(sv);
+    if (rows < 0)
         return ERROR_FUNCTION_FAILED;
+    sv->num_rows = rows;
 
     *view = (MSIVIEW *)sv;
 
