@@ -38,46 +38,38 @@ struct Keyword {
 
 #define MAX_TOKEN_LEN 11
 
+static const WCHAR ADD_W[] = { 'A','D','D',0 };
 static const WCHAR ALTER_W[] = { 'A','L','T','E','R',0 };
 static const WCHAR AND_W[] = { 'A','N','D',0 };
 static const WCHAR BY_W[] = { 'B','Y',0 };
 static const WCHAR CHAR_W[] = { 'C','H','A','R',0 };
 static const WCHAR CHARACTER_W[] = { 'C','H','A','R','A','C','T','E','R',0 };
 static const WCHAR CREATE_W[] = { 'C','R','E','A','T','E',0 };
-static const WCHAR CROSS_W[] = { 'C','R','O','S','S',0 };
 static const WCHAR DELETE_W[] = { 'D','E','L','E','T','E',0 };
 static const WCHAR DISTINCT_W[] = { 'D','I','S','T','I','N','C','T',0 };
 static const WCHAR FREE_W[] = { 'F','R','E','E',0 };
 static const WCHAR FROM_W[] = { 'F','R','O','M',0 };
-static const WCHAR FULL_W[] = { 'F','U','L','L',0 };
 static const WCHAR HOLD_W[] = { 'H','O','L','D',0 };
-static const WCHAR INNER_W[] = { 'I','N','N','E','R',0 };
 static const WCHAR INSERT_W[] = { 'I','N','S','E','R','T',0 };
 static const WCHAR INT_W[] = { 'I','N','T',0 };
 static const WCHAR INTEGER_W[] = { 'I','N','T','E','G','E','R',0 };
 static const WCHAR INTO_W[] = { 'I','N','T','O',0 };
 static const WCHAR IS_W[] = { 'I','S',0 };
-static const WCHAR JOIN_W[] = { 'J','O','I','N',0 };
 static const WCHAR KEY_W[] = { 'K','E','Y',0 };
-static const WCHAR LEFT_W[] = { 'L','E','F','T',0 };
 static const WCHAR LIKE_W[] = { 'L','I','K','E',0 };
 static const WCHAR LOCALIZABLE_W[] = { 'L','O','C','A','L','I','Z','A','B','L','E',0 };
 static const WCHAR LONG_W[] = { 'L','O','N','G',0 };
 static const WCHAR LONGCHAR_W[] = { 'L','O','N','G','C','H','A','R',0 };
-static const WCHAR NATURAL_W[] = { 'N','A','T','U','R','A','L',0 };
 static const WCHAR NOT_W[] = { 'N','O','T',0 };
 static const WCHAR NULL_W[] = { 'N','U','L','L',0 };
 static const WCHAR OBJECT_W[] = { 'O','B','J','E','C','T',0 };
 static const WCHAR OR_W[] = { 'O','R',0 };
 static const WCHAR ORDER_W[] = { 'O','R','D','E','R',0 };
-static const WCHAR OUTER_W[] = { 'O','U','T','E','R',0 };
 static const WCHAR PRIMARY_W[] = { 'P','R','I','M','A','R','Y',0 };
-static const WCHAR RIGHT_W[] = { 'R','I','G','H','T',0 };
 static const WCHAR SELECT_W[] = { 'S','E','L','E','C','T',0 };
 static const WCHAR SET_W[] = { 'S','E','T',0 };
 static const WCHAR SHORT_W[] = { 'S','H','O','R','T',0 };
 static const WCHAR TABLE_W[] = { 'T','A','B','L','E',0 };
-static const WCHAR TEMP_W[] = { 'T','E','M','P',0 };
 static const WCHAR TEMPORARY_W[] = { 'T','E','M','P','O','R','A','R','Y',0 };
 static const WCHAR UPDATE_W[] = { 'U','P','D','A','T','E',0 };
 static const WCHAR VALUES_W[] = { 'V','A','L','U','E','S',0 };
@@ -85,8 +77,10 @@ static const WCHAR WHERE_W[] = { 'W','H','E','R','E',0 };
 
 /*
 ** These are the keywords
+** They MUST be in alphabetical order
 */
 static const Keyword aKeywordTable[] = {
+  { ADD_W, TK_ADD },
   { ALTER_W, TK_ALTER },
   { AND_W, TK_AND },
   { BY_W, TK_BY },
@@ -255,7 +249,7 @@ int sqliteGetToken(const WCHAR *z, int *tokenType){
     case ',':
       *tokenType = TK_COMMA;
       return 1;
-    case '`': case '\'': case '"': {
+    case '`': case '\'': {
       int delim = z[0];
       for(i=1; z[i]; i++){
         if( z[i]==delim ){
