@@ -60,26 +60,56 @@ static UINT UPDATE_execute( struct tagMSIVIEW *view, MSIRECORD *record )
     MSIUPDATEVIEW *uv = (MSIUPDATEVIEW*)view;
     UINT i, r, col_count = 0, row_count = 0;
     MSIRECORD *values = NULL;
+    MSIRECORD *where = NULL;
     MSIVIEW *wv;
+    UINT cols_count, where_count;
+    column_info *col = uv->vals;
 
     TRACE("%p %p\n", uv, record );
 
+    /* extract the where markers from the record */
+    if (record)
+    {
+        r = MSI_RecordGetFieldCount(record);
+
+        for (i = 0; col; col = col->next)
+            i++;
+
+        cols_count = i;
+        where_count = r - i;
+
+        if (where_count > 0)
+        {
+            where = MSI_CreateRecord(where_count);
+
+            if (where)
+                for (i = 1; i <= where_count; i++)
+                    MSI_RecordCopyField(record, cols_count + i, where, i);
+        }
+    }
+
     wv = uv->wv;
     if( !wv )
-        return ERROR_FUNCTION_FAILED;
+    {
+        r = ERROR_FUNCTION_FAILED;
+        goto done;
+    }
 
-    r = wv->ops->execute( wv, 0 );
+    r = wv->ops->execute( wv, where );
     TRACE("tv execute returned %x\n", r);
     if( r )
-        return r;
+        goto done;
 
     r = wv->ops->get_dimensions( wv, &row_count, &col_count );
     if( r )
-        return r;
+        goto done;
 
     values = msi_query_merge_record( col_count, uv->vals, record );
     if (!values)
-        return ERROR_FUNCTION_FAILED;
+    {
+        r = ERROR_FUNCTION_FAILED;
+        goto done;
+    }
 
     for ( i=0; i<row_count; i++ )
     {
@@ -88,7 +118,9 @@ static UINT UPDATE_execute( struct tagMSIVIEW *view, MSIRECORD *record )
             break;
     }
 
-    msiobj_release( &values->hdr );
+done:
+    if ( where ) msiobj_release( &where->hdr );
+    if ( values ) msiobj_release( &values->hdr );
 
     return r;
 }
@@ -138,7 +170,7 @@ static UINT UPDATE_get_column_info( struct tagMSIVIEW *view,
 }
 
 static UINT UPDATE_modify( struct tagMSIVIEW *view, MSIMODIFY eModifyMode,
-                MSIRECORD *rec )
+                           MSIRECORD *rec, UINT row )
 {
     MSIUPDATEVIEW *uv = (MSIUPDATEVIEW*)view;
 
@@ -178,13 +210,18 @@ static const MSIVIEWOPS update_ops =
     NULL,
     NULL,
     NULL,
+    NULL,
     UPDATE_execute,
     UPDATE_close,
     UPDATE_get_dimensions,
     UPDATE_get_column_info,
     UPDATE_modify,
     UPDATE_delete,
-    UPDATE_find_matching_rows
+    UPDATE_find_matching_rows,
+    NULL,
+    NULL,
+    NULL,
+    NULL,
 };
 
 UINT UPDATE_CreateView( MSIDATABASE *db, MSIVIEW **view, LPCWSTR table,
