@@ -30,7 +30,7 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(msvcrt);
 
-unsigned char MSVCRT_mbctype[257];
+unsigned char MSVCRT_mbctype[257] = { 0 };
 static int g_mbcp_is_multibyte = 0;
 
 int MSVCRT___mb_cur_max = 1;
@@ -190,7 +190,7 @@ int CDECL _setmbcp(int cp)
   if (!GetCPInfo(newcp, &cpi))
   {
     WARN("Codepage %d not found\n", newcp);
-    msvcrt_set_errno(MSVCRT_EINVAL);
+    *MSVCRT__errno() = MSVCRT_EINVAL;
     return -1;
   }
 
@@ -442,6 +442,71 @@ unsigned char* CDECL _mbsncpy(unsigned char* dst, const unsigned char* src, MSVC
   return ret;
 }
 
+/*********************************************************************
+ *              _mbsnbcpy_s(MSVCRT.@)
+ * REMARKS
+ * Unlike _mbsnbcpy this function does not pad the rest of the dest
+ * string with 0
+ */
+int CDECL _mbsnbcpy_s(unsigned char* dst, MSVCRT_size_t size, const unsigned char* src, MSVCRT_size_t n)
+{
+    MSVCRT_size_t pos = 0;
+
+    if(!dst || size == 0)
+        return MSVCRT_EINVAL;
+    if(!src)
+    {
+        dst[0] = '\0';
+        return MSVCRT_EINVAL;
+    }
+    if(!n)
+        return 0;
+
+    if(g_mbcp_is_multibyte)
+    {
+        int is_lead = 0;
+        while (*src && n)
+        {
+            if(pos == size)
+            {
+                dst[0] = '\0';
+                return MSVCRT_ERANGE;
+            }
+            is_lead = (!is_lead && _ismbblead(*src));
+            n--;
+            dst[pos++] = *src++;
+        }
+
+        if (is_lead) /* if string ends with a lead, remove it */
+            dst[pos - 1] = 0;
+    }
+    else
+    {
+        while (n)
+        {
+            n--;
+            if(pos == size)
+            {
+                dst[0] = '\0';
+                return MSVCRT_ERANGE;
+            }
+
+            if(!(*src)) break;
+            dst[pos++] = *src++;
+        }
+    }
+
+    if(pos < size)
+        dst[pos] = '\0';
+    else
+    {
+        dst[0] = '\0';
+        return MSVCRT_ERANGE;
+    }
+
+    return 0;
+}
+
 /*********************************************************************
  *              _mbsnbcpy(MSVCRT.@)
  * REMARKS
