@@ -109,8 +109,6 @@ static HRESULT WINAPI xmlcf_CreateInstance(
         return r;
 
     r = IUnknown_QueryInterface( punk, riid, ppobj );
-    if (FAILED(r))
-        return r;
     IUnknown_Release( punk );
     return r;
 }
@@ -134,6 +132,8 @@ static const struct IClassFactoryVtbl xmlcf_vtbl =
 
 static xmlcf domdoccf = { &xmlcf_vtbl, DOMDocument_create };
 static xmlcf schemacf = { &xmlcf_vtbl, SchemaCache_create };
+static xmlcf xmldoccf = { &xmlcf_vtbl, XMLDocument_create };
+static xmlcf saxreadcf = { &xmlcf_vtbl, SAXXMLReader_create };
 
 /******************************************************************
  *		DllGetClassObject (MSXML3.@)
@@ -147,11 +147,27 @@ HRESULT WINAPI DllGetClassObject( REFCLSID rclsid, REFIID iid, LPVOID *ppv )
     if( IsEqualCLSID( rclsid, &CLSID_DOMDocument ) ||   /* Version indep. v 2.x */
         IsEqualCLSID( rclsid, &CLSID_DOMDocument2 ) ||  /* Version indep. v 3.0 */
         IsEqualCLSID( rclsid, &CLSID_DOMDocument30 ) )  /* Version dep.   v 3.0 */
+    {
         cf = (IClassFactory*) &domdoccf.lpVtbl;
-
+    }
     else if( IsEqualCLSID( rclsid, &CLSID_XMLSchemaCache ) ||
              IsEqualCLSID( rclsid, &CLSID_XMLSchemaCache30 ) )
+    {
         cf = (IClassFactory*) &schemacf.lpVtbl;
+    }
+    else if( IsEqualCLSID( rclsid, &CLSID_XMLDocument ) )
+    {
+        cf = (IClassFactory*) &xmldoccf.lpVtbl;
+    }
+    else if( IsEqualCLSID( rclsid, &CLSID_FreeThreadedDOMDocument ) )
+    {
+        cf = (IClassFactory*) &domdoccf.lpVtbl;
+    }
+    else if( IsEqualCLSID( rclsid, &CLSID_SAXXMLReader) ||
+             IsEqualCLSID( rclsid, &CLSID_SAXXMLReader30 ))
+    {
+        cf = (IClassFactory*) &saxreadcf.lpVtbl;
+    }
 
     if ( !cf )
         return CLASS_E_CLASSNOTAVAILABLE;
