@@ -39,7 +39,6 @@
 #endif
 
 static VOID     (WINAPI *pRtlInitUnicodeString)( PUNICODE_STRING, LPCWSTR );
-static VOID     (WINAPI *pRtlFreeUnicodeString)(PUNICODE_STRING);
 static NTSTATUS (WINAPI *pNtCreateMailslotFile)( PHANDLE, ULONG, POBJECT_ATTRIBUTES, PIO_STATUS_BLOCK,
                                        ULONG, ULONG, ULONG, PLARGE_INTEGER );
 static NTSTATUS (WINAPI *pNtReadFile)(HANDLE hFile, HANDLE hEvent,
@@ -66,6 +65,7 @@ static inline BOOL is_signaled( HANDLE obj )
 }
 
 #define PIPENAME "\\\\.\\pipe\\ntdll_tests_file.c"
+#define TEST_BUF_LEN 3
 
 static BOOL create_pipe( HANDLE *read, HANDLE *write, ULONG flags, ULONG size )
 {
@@ -403,7 +403,8 @@ static void nt_mailslot_test(void)
 
     pRtlInitUnicodeString(&str, buffer1);
     InitializeObjectAttributes(&attr, &str, OBJ_CASE_INSENSITIVE, 0, NULL);
-    DesiredAccess = CreateOptions = MailslotQuota = MaxMessageSize = 0;
+    CreateOptions = MailslotQuota = MaxMessageSize = 0;
+    DesiredAccess = GENERIC_READ;
 
     /*
      * Check for NULL pointer handling
@@ -411,15 +412,20 @@ static void nt_mailslot_test(void)
     rc = pNtCreateMailslotFile(NULL, DesiredAccess,
          &attr, &IoStatusBlock, CreateOptions, MailslotQuota, MaxMessageSize,
          &TimeOut);
-    ok( rc == STATUS_ACCESS_VIOLATION, "rc = %x not c0000005 STATUS_ACCESS_VIOLATION\n", rc);
+    ok( rc == STATUS_ACCESS_VIOLATION ||
+        rc == STATUS_INVALID_PARAMETER, /* win2k3 */
+        "rc = %x not STATUS_ACCESS_VIOLATION or STATUS_INVALID_PARAMETER\n", rc);
 
     /*
      * Test to see if the Timeout can be NULL
      */
+    hslot = (HANDLE)0xdeadbeef;
     rc = pNtCreateMailslotFile(&hslot, DesiredAccess,
          &attr, &IoStatusBlock, CreateOptions, MailslotQuota, MaxMessageSize,
          NULL);
-    ok( rc == STATUS_SUCCESS, "rc = %x not STATUS_SUCCESS\n", rc);
+    ok( rc == STATUS_SUCCESS ||
+        rc == STATUS_INVALID_PARAMETER, /* win2k3 */
+        "rc = %x not STATUS_SUCCESS or STATUS_INVALID_PARAMETER\n", rc);
     ok( hslot != 0, "Handle is invalid\n");
 
     if  ( rc == STATUS_SUCCESS ) rc = pNtClose(hslot);
@@ -451,7 +457,9 @@ static void nt_mailslot_test(void)
     rc = pNtCreateMailslotFile(&hslot, DesiredAccess,
          &attr, &IoStatusBlock, CreateOptions, MailslotQuota, MaxMessageSize,
          &TimeOut);
-    ok( rc == STATUS_OBJECT_PATH_SYNTAX_BAD, "rc = %x not c000003b STATUS_OBJECT_PATH_SYNTAX_BAD\n", rc);
+    ok( rc == STATUS_OBJECT_PATH_SYNTAX_BAD ||
+        rc == STATUS_INVALID_PARAMETER,
+        "rc = %x not STATUS_OBJECT_PATH_SYNTAX_BAD or STATUS_INVALID_PARAMETER\n", rc);
 
     if  (rc == STATUS_SUCCESS) pNtClose(hslot);
 
@@ -462,13 +470,11 @@ static void nt_mailslot_test(void)
     rc = pNtCreateMailslotFile(&hslot, DesiredAccess,
          &attr, &IoStatusBlock, CreateOptions, MailslotQuota, MaxMessageSize,
          &TimeOut);
-    ok( rc == STATUS_SUCCESS, "Create MailslotFile failed rc = %x %u\n", rc, GetLastError());
+    ok( rc == STATUS_SUCCESS, "Create MailslotFile failed rc = %x\n", rc);
     ok( hslot != 0, "Handle is invalid\n");
 
     rc = pNtClose(hslot);
     ok( rc == STATUS_SUCCESS, "NtClose failed\n");
-
-    pRtlFreeUnicodeString(&str);
 }
 
 static void test_iocp_setcompletion(HANDLE h)
@@ -528,7 +534,7 @@ static void test_iocp_fileio(HANDLE h)
     if (hPipeClt != INVALID_HANDLE_VALUE)
     {
         OVERLAPPED o = {0,};
-        BYTE buf[3];
+        BYTE send_buf[TEST_BUF_LEN], recv_buf[TEST_BUF_LEN];
         DWORD read;
         long count;
 
@@ -536,12 +542,14 @@ static void test_iocp_fileio(HANDLE h)
         ok( res == STATUS_SUCCESS, "NtSetInformationFile failed: %x\n", res );
         ok( U(iosb).Status == STATUS_SUCCESS, "iosb.Status invalid: %x\n", U(iosb).Status );
 
+        memset( send_buf, 0, TEST_BUF_LEN );
+        memset( recv_buf, 0xde, TEST_BUF_LEN );
         count = get_pending_msgs(h);
         ok( !count, "Unexpected msg count: %ld\n", count );
-        ReadFile( hPipeSrv, buf, 3, &read, &o);
+        ReadFile( hPipeSrv, recv_buf, TEST_BUF_LEN, &read, &o);
         count = get_pending_msgs(h);
         ok( !count, "Unexpected msg count: %ld\n", count );
-        WriteFile( hPipeClt, buf, 3, &read, NULL );
+        WriteFile( hPipeClt, send_buf, TEST_BUF_LEN, &read, NULL );
 
         if (get_msg(h))
         {
@@ -549,14 +557,17 @@ static void test_iocp_fileio(HANDLE h)
             ok( ioSb.Information == 3, "Invalid ioSb.Information: %ld\n", ioSb.Information );
             ok( U(ioSb).Status == STATUS_SUCCESS, "Invalid ioSb.Status: %x\n", U(ioSb).Status);
             ok( completionValue == (ULONG_PTR)&o, "Invalid completion value: %lx\n", completionValue );
+            ok( !memcmp( send_buf, recv_buf, TEST_BUF_LEN ), "Receive buffer (%x %x %x) did not match send buffer (%x %x %x)\n", recv_buf[0], recv_buf[1], recv_buf[2], send_buf[0], send_buf[1], send_buf[2] );
         }
         count = get_pending_msgs(h);
         ok( !count, "Unexpected msg count: %ld\n", count );
 
-        WriteFile( hPipeClt, buf, 2, &read, NULL );
+        memset( send_buf, 0, TEST_BUF_LEN );
+        memset( recv_buf, 0xde, TEST_BUF_LEN );
+        WriteFile( hPipeClt, send_buf, 2, &read, NULL );
         count = get_pending_msgs(h);
         ok( !count, "Unexpected msg count: %ld\n", count );
-        ReadFile( hPipeSrv, buf, 2, &read, &o);
+        ReadFile( hPipeSrv, recv_buf, 2, &read, &o);
         count = get_pending_msgs(h);
         ok( count == 1, "Unexpected msg count: %ld\n", count );
         if (get_msg(h))
@@ -565,9 +576,10 @@ static void test_iocp_fileio(HANDLE h)
             ok( ioSb.Information == 2, "Invalid ioSb.Information: %ld\n", ioSb.Information );
             ok( U(ioSb).Status == STATUS_SUCCESS, "Invalid ioSb.Status: %x\n", U(ioSb).Status);
             ok( completionValue == (ULONG_PTR)&o, "Invalid completion value: %lx\n", completionValue );
+            ok( !memcmp( send_buf, recv_buf, 2 ), "Receive buffer (%x %x) did not match send buffer (%x %x)\n", recv_buf[0], recv_buf[1], send_buf[0], send_buf[1] );
         }
 
-        ReadFile( hPipeSrv, buf, sizeof(buf), &read, &o);
+        ReadFile( hPipeSrv, recv_buf, TEST_BUF_LEN, &read, &o);
         CloseHandle( hPipeSrv );
         count = get_pending_msgs(h);
         ok( count == 1, "Unexpected msg count: %ld\n", count );
@@ -611,7 +623,6 @@ START_TEST(file)
         return;
     }
 
-    pRtlFreeUnicodeString   = (void *)GetProcAddress(hntdll, "RtlFreeUnicodeString");
     pRtlInitUnicodeString   = (void *)GetProcAddress(hntdll, "RtlInitUnicodeString");
     pNtCreateMailslotFile   = (void *)GetProcAddress(hntdll, "NtCreateMailslotFile");
     pNtReadFile             = (void *)GetProcAddress(hntdll, "NtReadFile");
