@@ -237,13 +237,13 @@ static HRESULT WINAPI BaseMemAllocator_Commit(IMemAllocator * iface)
     {
         if (!This->pProps)
             hr = VFW_E_SIZENOTSET;
-        else if (This->bCommitted)
-            hr = S_OK;
-        else if (This->bDecommitQueued)
+        else if (This->bDecommitQueued && This->bCommitted)
         {
             This->bDecommitQueued = FALSE;
             hr = S_OK;
         }
+        else if (This->bCommitted)
+            hr = S_OK;
         else
         {
             if (!(This->hSemWaiting = CreateSemaphoreW(NULL, This->pProps->cBuffers, This->pProps->cBuffers, NULL)))
@@ -586,7 +586,10 @@ static HRESULT WINAPI StdMediaSample2_SetSyncPoint(IMediaSample2 * iface, BOOL b
 
     TRACE("(%s)\n", bIsSyncPoint ? "TRUE" : "FALSE");
 
-    This->props.dwSampleFlags = (This->props.dwSampleFlags & ~AM_SAMPLE_SPLICEPOINT) | bIsSyncPoint ? AM_SAMPLE_SPLICEPOINT : 0;
+    if (bIsSyncPoint)
+        This->props.dwSampleFlags |= AM_SAMPLE_SPLICEPOINT;
+    else
+        This->props.dwSampleFlags &= ~AM_SAMPLE_SPLICEPOINT;
 
     return S_OK;
 }
@@ -606,7 +609,10 @@ static HRESULT WINAPI StdMediaSample2_SetPreroll(IMediaSample2 * iface, BOOL bIs
 
     TRACE("(%s)\n", bIsPreroll ? "TRUE" : "FALSE");
 
-    This->props.dwSampleFlags = (This->props.dwSampleFlags & ~AM_SAMPLE_PREROLL) | bIsPreroll ? AM_SAMPLE_PREROLL : 0;
+    if (bIsPreroll)
+        This->props.dwSampleFlags |= AM_SAMPLE_PREROLL;
+    else
+        This->props.dwSampleFlags &= ~AM_SAMPLE_PREROLL;
 
     return S_OK;
 }
