@@ -855,12 +855,24 @@ static struct regsvr_coclass const coclass_list[] = {
 	"quartz.dll",
 	"Both"
     },
+    {   &CLSID_NullRenderer,
+        "Null Renderer",
+        NULL,
+        "quartz.dll",
+        "Both"
+    },
     {   &CLSID_VideoRenderer,
 	"Video Renderer",
 	NULL,
 	"quartz.dll",
 	"Both"
     },
+    {   &CLSID_VideoRendererDefault,
+        "Default Video Renderer",
+        NULL,
+        "quartz.dll",
+        "Both"
+    },
     {   &CLSID_ACMWrapper,
 	"ACM wrapper",
 	NULL,
@@ -962,7 +974,7 @@ static struct regsvr_mediatype_parsing const mediatype_parsing_list[] = {
     {	&MEDIATYPE_Stream,
 	&MEDIASUBTYPE_MPEG1Audio,
 	{   "0, 2, FFE0, FFE0",
-	    "0, 10, FFFFFFFF000000000000, 494433030080808080",
+            "0, 10, FFFFFF00000080808080, 494433000000000000",
 	    NULL }
     },
     {	&MEDIATYPE_Stream,
@@ -1063,6 +1075,18 @@ static struct regsvr_filter const filter_list[] = {
             { 0xFFFFFFFF },
         }
     },
+    {   &CLSID_NullRenderer,
+        &CLSID_LegacyAmFilterCategory,
+        {'N','u','l','l',' ','R','e','n','d','e','r','e','r',0},
+        0x200000,
+        {   {   REG_PINFLAG_B_RENDERER,
+                {   { &MEDIATYPE_NULL, &GUID_NULL },
+                    { NULL }
+                },
+            },
+            { 0xFFFFFFFF },
+        }
+    },
     {   &CLSID_VideoRenderer,
 	&CLSID_LegacyAmFilterCategory,
 	{'V','i','d','e','o',' ','R','e','n','d','e','r','e','r',0},
@@ -1075,6 +1099,31 @@ static struct regsvr_filter const filter_list[] = {
 	    { 0xFFFFFFFF },
 	}
     },
+    {   &CLSID_VideoRendererDefault,
+        &CLSID_LegacyAmFilterCategory,
+        {'V','i','d','e','o',' ','R','e','n','d','e','r','e','r',0},
+        0x800000,
+        {   {   REG_PINFLAG_B_RENDERER,
+                {   { &MEDIATYPE_Video, &GUID_NULL },
+                    { NULL }
+                },
+            },
+            { 0xFFFFFFFF },
+        }
+    },
+    {   &CLSID_DSoundRender,
+        &CLSID_LegacyAmFilterCategory,
+        {'A','u','d','i','o',' ','R','e','n','d','e','r','e','r',0},
+        0x800000,
+        {   {   REG_PINFLAG_B_RENDERER,
+                {   { &MEDIATYPE_Audio, &MEDIASUBTYPE_PCM },
+/*                  { &MEDIATYPE_Audio, &MEDIASUBTYPE_IEEE_FLOAT }, */
+                    { NULL }
+                },
+            },
+            { 0xFFFFFFFF },
+        }
+    },
     {   &CLSID_AVIDec,
 	&CLSID_LegacyAmFilterCategory,
 	{'A','V','I',' ','D','e','c','o','m','p','r','e','s','s','o','r',0},
