@@ -29,7 +29,6 @@
 #include "windef.h"
 #include "winbase.h"
 #include "winerror.h"
-#include "winreg.h"
 
 #include "rpc.h"
 
@@ -268,6 +267,9 @@ RPC_STATUS WINAPI TowerExplode(
     const twr_uuid_floor_t *object_floor;
     const twr_uuid_floor_t *syntax_floor;
 
+    TRACE("(%p, %p, %p, %p, %p, %p)\n", tower, object, syntax, protseq,
+          endpoint, address);
+
     if (protseq)
         *protseq = NULL;
     if (endpoint)
@@ -337,6 +339,9 @@ RPC_STATUS WINAPI TowerConstruct(
     twr_uuid_floor_t *object_floor;
     twr_uuid_floor_t *syntax_floor;
 
+    TRACE("(%p, %p, %s, %s, %s, %p)\n", object, syntax, debugstr_a(protseq),
+          debugstr_a(endpoint), debugstr_a(address), tower);
+
     *tower = NULL;
 
     status = RpcTransport_GetTopOfTower(NULL, &tower_size, protseq, address, endpoint);
