@@ -36,6 +36,7 @@
 #include "sane_i.h"
 #include "wine/debug.h"
 #include "resource.h"
+#include "wine/unicode.h"
 
 #ifdef SONAME_LIBSANE
 
@@ -370,10 +371,8 @@ static int create_item(HDC hdc, const SANE_Option_Descriptor *opt,
 
 
 static LPDLGTEMPLATEW create_options_page(HDC hdc, int *from_index,
-                                          BOOL split_tabs)
+                                          SANE_Int optcount, BOOL split_tabs)
 {
-    SANE_Status rc;
-    SANE_Int optcount;
     int i;
     INT y = 2;
     LPDLGTEMPLATEW tpl = NULL;
@@ -385,15 +384,6 @@ static LPDLGTEMPLATEW create_options_page(HDC hdc, int *from_index,
     int group_offset = -1;
     INT control_count = 0;
 
-    rc = psane_control_option(activeDS.deviceHandle, 0, SANE_ACTION_GET_VALUE,
-            &optcount, NULL);
-
-    if (rc != SANE_STATUS_GOOD)
-    {
-        ERR("Unable to read number of options\n");
-        return NULL;
-    }
-
     for (i = *from_index; i < optcount; i++)
     {
         LPDLGITEMTEMPLATEW item_tpl = NULL;
@@ -405,6 +395,8 @@ static LPDLGTEMPLATEW create_options_page(HDC hdc, int *from_index,
         int hold_for_group = 0;
 
         opt = psane_get_option_descriptor(activeDS.deviceHandle, i);
+        if (!opt)
+            continue;
         if (opt->type == SANE_TYPE_GROUP && split_tabs)
         {
             if (control_len > 0)
@@ -418,6 +410,8 @@ static LPDLGTEMPLATEW create_options_page(HDC hdc, int *from_index,
                 return NULL;
             }
         }
+        if (!SANE_OPTION_IS_ACTIVE (opt->cap))
+            continue;
 
         len = create_item(hdc, opt, ID_BASE + i, &item_tpl, y, &x, &count);
 
@@ -540,12 +534,17 @@ BOOL DoScannerUI(void)
     memset(&psp,0,sizeof(psp));
     rc = psane_control_option(activeDS.deviceHandle, 0, SANE_ACTION_GET_VALUE,
             &optcount, NULL);
+    if (rc != SANE_STATUS_GOOD)
+    {
+        ERR("Unable to read number of options\n");
+        return FALSE;
+    }
 
     while (index < optcount)
     {
         const SANE_Option_Descriptor *opt;
         psp[page_count].u.pResource = create_options_page(hdc, &index,
-                TRUE);
+                optcount, TRUE);
         opt = psane_get_option_descriptor(activeDS.deviceHandle, index);
 
         if (opt->type == SANE_TYPE_GROUP)
@@ -599,6 +598,7 @@ BOOL DoScannerUI(void)
         HeapFree(GetProcessHeap(),0,(LPBYTE)psp[index].u.pResource);
         HeapFree(GetProcessHeap(),0,(LPBYTE)psp[index].pszTitle);
     }
+    HeapFree(GetProcessHeap(),0,szCaption);
     
     if (psrc == IDOK)
         return TRUE;
@@ -609,14 +609,13 @@ BOOL DoScannerUI(void)
 static void UpdateRelevantEdit(HWND hwnd, const SANE_Option_Descriptor *opt, 
         int index, int position)
 {
-    CHAR buffer[244];
+    WCHAR buffer[244];
     HWND edit_w;
-    CHAR unit[20];
-
-    LoadStringA(SANE_instance, opt->unit, unit,20);
+    int len;
 
     if (opt->type == SANE_TYPE_INT)
     {
+        static const WCHAR formatW[] = {'%','i',0};
         INT si;
 
         if (opt->constraint.range->quant)
@@ -624,11 +623,11 @@ static void UpdateRelevantEdit(HWND hwnd, const SANE_Option_Descriptor *opt,
         else
             si = position;
 
-        sprintf(buffer,"%i %s",si,unit);
-
+        len = sprintfW( buffer, formatW, si );
     }
     else if  (opt->type == SANE_TYPE_FIXED)
     {
+        static const WCHAR formatW[] = {'%','f',0};
         double s_quant, dd;
 
         s_quant = SANE_UNFIX(opt->constraint.range->quant);
@@ -638,14 +637,15 @@ static void UpdateRelevantEdit(HWND hwnd, const SANE_Option_Descriptor *opt,
         else
             dd = position * 0.01;
 
-        sprintf(buffer,"%f %s",dd,unit);
+        len = sprintfW( buffer, formatW, dd );
     }
-    else
-        buffer[0] = 0;
+    else return;
+
+    buffer[len++] = ' ';
+    LoadStringW( SANE_instance, opt->unit, buffer + len, sizeof(buffer)/sizeof(WCHAR) - len );
 
     edit_w = GetDlgItem(hwnd,index+ID_BASE+ID_EDIT_BASE);
-    if (edit_w && buffer[0])
-        SetWindowTextA(edit_w,buffer);
+    if (edit_w) SetWindowTextW(edit_w,buffer);
 
 }
 
@@ -745,6 +745,11 @@ static INT_PTR InitializeDialog(HWND hwnd)
 
     rc = psane_control_option(activeDS.deviceHandle, 0, SANE_ACTION_GET_VALUE,
             &optcount, NULL);
+    if (rc != SANE_STATUS_GOOD)
+    {
+        ERR("Unable to read number of options\n");
+        return FALSE;
+    }
 
     for ( i = 1; i < optcount; i++)
     {
@@ -982,7 +987,6 @@ static INT_PTR CALLBACK DialogProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM
             return InitializeDialog(hwndDlg);
         case WM_HSCROLL:
             return ProcessScroll(hwndDlg, wParam, lParam);
-            break;
         case WM_NOTIFY:
             {
                 LPPSHNOTIFY psn = (LPPSHNOTIFY)lParam;
