@@ -3,7 +3,7 @@
  *
  *	Copyright 1998,1999	<juergen.schmied@debitel.net>
  *
- * This is the view visualizing the data provied by the shellfolder.
+ * This is the view visualizing the data provided by the shellfolder.
  * No direct access to data from pidls should be done from here.
  *
  * This library is free software; you can redistribute it and/or
@@ -21,9 +21,9 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  *
  * FIXME: The order by part of the background context menu should be
- * buily according to the columns shown.
+ * built according to the columns shown.
  *
- * FIXME: Load/Save the view state from/into the stream provied by
+ * FIXME: Load/Save the view state from/into the stream provided by
  * the ShellBrowser
  *
  * FIXME: CheckToolbar: handle the "new folder" and "folder up" button
@@ -768,9 +768,21 @@ static void ShellView_MergeFileMenu(IShellViewImpl * This, HMENU hSubMenu)
 
 	if(hSubMenu)
 	{ /*insert This item at the beginning of the menu */
-	  _InsertMenuItem(hSubMenu, 0, TRUE, 0, MFT_SEPARATOR, NULL, MFS_ENABLED);
-	  _InsertMenuItem(hSubMenu, 0, TRUE, IDM_MYFILEITEM, MFT_STRING, "dummy45", MFS_ENABLED);
+            MENUITEMINFOA mii;
 
+            mii.cbSize = sizeof(mii);
+            mii.fMask = MIIM_ID | MIIM_TYPE;
+            mii.wID = 0;
+            mii.fType = MFT_SEPARATOR;
+            InsertMenuItemA(hSubMenu, 0, TRUE, &mii);
+
+            mii.cbSize = sizeof(mii);
+            mii.fMask = MIIM_ID | MIIM_TYPE | MIIM_STATE;
+            mii.dwTypeData = (LPSTR)"dummy45";
+            mii.fState = MFS_ENABLED;
+            mii.wID = IDM_MYFILEITEM;
+            mii.fType = MFT_STRING;
+            InsertMenuItemA(hSubMenu, 0, TRUE, &mii);
 	}
 	TRACE("--\n");
 }
@@ -788,9 +800,13 @@ static void ShellView_MergeViewMenu(IShellViewImpl * This, HMENU hSubMenu)
 	  MENUITEMINFOA mii;
 	  static char view[] = "View";
 
-	  _InsertMenuItem(hSubMenu, FCIDM_MENU_VIEW_SEP_OPTIONS, FALSE, 0, MFT_SEPARATOR, NULL, MFS_ENABLED);
+          ZeroMemory(&mii, sizeof(mii));
+          mii.cbSize = sizeof(mii);
+          mii.fMask = MIIM_ID | MIIM_TYPE;
+          mii.wID = 0;
+          mii.fType = MFT_SEPARATOR;
+          InsertMenuItemA(hSubMenu, FCIDM_MENU_VIEW_SEP_OPTIONS, FALSE, &mii);
 
-	  ZeroMemory(&mii, sizeof(mii));
 	  mii.cbSize = sizeof(mii);
 	  mii.fMask = MIIM_SUBMENU | MIIM_TYPE | MIIM_DATA;
 	  mii.fType = MFT_STRING;
@@ -1541,6 +1557,15 @@ static LRESULT ShellView_OnNotify(IShellViewImpl * This, UINT CtlID, LPNMHDR lpn
 		IShellView_Refresh((IShellView*)This);
               }
 
+	      else if(plvKeyDown->wVKey == VK_BACK)
+	      {
+		LPSHELLBROWSER lpSb;
+		if((lpSb = (LPSHELLBROWSER)SendMessageW(This->hWndParent, CWM_GETISHELLBROWSER, 0, 0)))
+		{
+		  IShellBrowser_BrowseObject(lpSb, NULL, SBSP_PARENT);
+		}
+	      }
+
               else
 		FIXME("LVN_KEYDOWN key=0x%08x\n",plvKeyDown->wVKey);
 	    }
@@ -1557,7 +1582,7 @@ static LRESULT ShellView_OnNotify(IShellViewImpl * This, UINT CtlID, LPNMHDR lpn
 * ShellView_OnChange()
 */
 
-static LRESULT ShellView_OnChange(IShellViewImpl * This, LPITEMIDLIST * Pidls, LONG wEventId)
+static LRESULT ShellView_OnChange(IShellViewImpl * This, const LPCITEMIDLIST * Pidls, LONG wEventId)
 {
 
 	TRACE("(%p)(%p,%p,0x%08x)\n", This, Pidls[0], Pidls[1], wEventId);
@@ -1610,7 +1635,7 @@ static LRESULT CALLBACK ShellView_WndProc(HWND hWnd, UINT uMessage, WPARAM wPara
 					GET_WM_COMMAND_ID(wParam, lParam),
 					GET_WM_COMMAND_CMD(wParam, lParam),
 					GET_WM_COMMAND_HWND(wParam, lParam));
-	  case SHV_CHANGE_NOTIFY: return ShellView_OnChange(pThis, (LPITEMIDLIST*)wParam, (LONG)lParam);
+	  case SHV_CHANGE_NOTIFY: return ShellView_OnChange(pThis, (const LPCITEMIDLIST*)wParam, (LONG)lParam);
 
 	  case WM_CONTEXTMENU:  ShellView_DoContextMenu(pThis, LOWORD(lParam), HIWORD(lParam), FALSE);
 	                        return 0;
