@@ -223,7 +223,7 @@ static int get_dib_width_bytes( int width, int depth )
  */
 static int bitmap_info_size( const BITMAPINFO * info, WORD coloruse )
 {
-    int colors;
+    int colors, masks = 0;
 
     if (info->bmiHeader.biSize == sizeof(BITMAPCOREHEADER))
     {
@@ -239,7 +239,8 @@ static int bitmap_info_size( const BITMAPINFO * info, WORD coloruse )
                 colors = 256;
         if (!colors && (info->bmiHeader.biBitCount <= 8))
             colors = 1 << info->bmiHeader.biBitCount;
-        return sizeof(BITMAPINFOHEADER) + colors *
+        if (info->bmiHeader.biCompression == BI_BITFIELDS) masks = 3;
+        return sizeof(BITMAPINFOHEADER) + masks * sizeof(DWORD) + colors *
                ((coloruse == DIB_RGB_COLORS) ? sizeof(RGBQUAD) : sizeof(WORD));
     }
 }
@@ -384,11 +385,10 @@ static ICONCACHE* CURSORICON_FindCache(HICON hIcon)
     ICONCACHE *ptr;
     ICONCACHE *pRet=NULL;
     BOOL IsFound = FALSE;
-    int count;
 
     EnterCriticalSection( &IconCrst );
 
-    for (count = 0, ptr = IconAnchor; ptr != NULL && !IsFound; ptr = ptr->next, count++ )
+    for (ptr = IconAnchor; ptr != NULL && !IsFound; ptr = ptr->next)
     {
         if ( hIcon == ptr->hIcon )
         {
@@ -660,48 +660,26 @@ static CURSORICONFILEDIRENTRY *CURSORICON_FindBestIconFile( CURSORICONFILEDIR *d
     return &dir->idEntries[n];
 }
 
-/**********************************************************************
- *		CreateIconFromResourceEx (USER32.@)
- *
- * FIXME: Convert to mono when cFlag is LR_MONOCHROME. Do something
- *        with cbSize parameter as well.
- */
-HICON WINAPI CreateIconFromResourceEx( LPBYTE bits, UINT cbSize,
-                                       BOOL bIcon, DWORD dwVersion,
-                                       INT width, INT height,
-                                       UINT cFlag )
+static HICON CURSORICON_CreateIconFromBMI( BITMAPINFO *bmi,
+					   POINT16 hotspot, BOOL bIcon,
+					   DWORD dwVersion,
+					   INT width, INT height,
+					   UINT cFlag )
 {
     HGLOBAL16 hObj;
     static HDC hdcMem;
     int sizeAnd, sizeXor;
     HBITMAP hAndBits = 0, hXorBits = 0; /* error condition for later */
     BITMAP bmpXor, bmpAnd;
-    POINT16 hotspot;
-    BITMAPINFO *bmi;
     BOOL DoStretch;
     INT size;
 
-    hotspot.x = ICON_HOTSPOT;
-    hotspot.y = ICON_HOTSPOT;
-
-    TRACE_(cursor)("%p (%u bytes), ver %08x, %ix%i %s %s\n",
-                   bits, cbSize, dwVersion, width, height,
-                                  bIcon ? "icon" : "cursor", (cFlag & LR_MONOCHROME) ? "mono" : "" );
     if (dwVersion == 0x00020000)
     {
         FIXME_(cursor)("\t2.xx resources are not supported\n");
         return 0;
     }
 
-    if (bIcon)
-        bmi = (BITMAPINFO *)bits;
-    else /* get the hotspot */
-    {
-        POINT16 *pt = (POINT16 *)bits;
-        hotspot = *pt;
-        bmi = (BITMAPINFO *)(pt + 1);
-    }
-
     /* Check bitmap header */
 
     if ( (bmi->bmiHeader.biSize != sizeof(BITMAPCOREHEADER)) &&
@@ -872,6 +850,41 @@ HICON WINAPI CreateIconFromResourceEx( LPBYTE bits, UINT cbSize,
 }
 
 
+/**********************************************************************
+ *		CreateIconFromResourceEx (USER32.@)
+ *
+ * FIXME: Convert to mono when cFlag is LR_MONOCHROME. Do something
+ *        with cbSize parameter as well.
+ */
+HICON WINAPI CreateIconFromResourceEx( LPBYTE bits, UINT cbSize,
+                                       BOOL bIcon, DWORD dwVersion,
+                                       INT width, INT height,
+                                       UINT cFlag )
+{
+    POINT16 hotspot;
+    BITMAPINFO *bmi;
+
+    hotspot.x = ICON_HOTSPOT;
+    hotspot.y = ICON_HOTSPOT;
+
+    TRACE_(cursor)("%p (%u bytes), ver %08x, %ix%i %s %s\n",
+                   bits, cbSize, dwVersion, width, height,
+                                  bIcon ? "icon" : "cursor", (cFlag & LR_MONOCHROME) ? "mono" : "" );
+
+    if (bIcon)
+        bmi = (BITMAPINFO *)bits;
+    else /* get the hotspot */
+    {
+        POINT16 *pt = (POINT16 *)bits;
+        hotspot = *pt;
+        bmi = (BITMAPINFO *)(pt + 1);
+    }
+
+    return CURSORICON_CreateIconFromBMI( bmi, hotspot, bIcon, dwVersion,
+					 width, height, cFlag );
+}
+
+
 /**********************************************************************
  *		CreateIconFromResource (USER32.@)
  */
@@ -891,6 +904,7 @@ static HICON CURSORICON_LoadFromFile( LPCWSTR filename,
     DWORD filesize = 0;
     HICON hIcon = 0;
     LPBYTE bits;
+    POINT16 hotspot;
 
     TRACE("loading %s\n", debugstr_w( filename ));
 
@@ -926,8 +940,11 @@ static HICON CURSORICON_LoadFromFile( LPCWSTR filename,
     if ( entry->dwDIBOffset + entry->dwDIBSize > filesize )
         goto end;
 
-    hIcon = CreateIconFromResourceEx( &bits[entry->dwDIBOffset], entry->dwDIBSize,
-                                      !fCursor, 0x00030000, width, height, loadflags );
+    hotspot.x = entry->xHotspot;
+    hotspot.y = entry->yHotspot;
+    hIcon = CURSORICON_CreateIconFromBMI( (BITMAPINFO *)&bits[entry->dwDIBOffset],
+					  hotspot, !fCursor, 0x00030000,
+					  width, height, loadflags );
 end:
     TRACE("loaded %s -> %p\n", debugstr_w( filename ), hIcon );
     UnmapViewOfFile( bits );
@@ -1790,9 +1807,10 @@ BOOL WINAPI GetIconInfo(HICON hIcon, PICONINFO iconinfo)
  */
 HICON WINAPI CreateIconIndirect(PICONINFO iconinfo)
 {
-    BITMAP bmpXor,bmpAnd;
+    DIBSECTION bmpXor;
+    BITMAP bmpAnd;
     HICON16 hObj;
-    int	sizeXor,sizeAnd;
+    int xor_objsize = 0, sizeXor = 0, sizeAnd, planes, bpp;
 
     TRACE("color %p, mask %p, hotspot %ux%u, fIcon %d\n",
            iconinfo->hbmColor, iconinfo->hbmMask,
@@ -1800,19 +1818,24 @@ HICON WINAPI CreateIconIndirect(PICONINFO iconinfo)
 
     if (!iconinfo->hbmMask) return 0;
 
+    planes = GetDeviceCaps( screen_dc, PLANES );
+    bpp = GetDeviceCaps( screen_dc, BITSPIXEL );
+
     if (iconinfo->hbmColor)
     {
-        GetObjectW( iconinfo->hbmColor, sizeof(bmpXor), &bmpXor );
+        xor_objsize = GetObjectW( iconinfo->hbmColor, sizeof(bmpXor), &bmpXor );
         TRACE("color: width %d, height %d, width bytes %d, planes %u, bpp %u\n",
-               bmpXor.bmWidth, bmpXor.bmHeight, bmpXor.bmWidthBytes,
-               bmpXor.bmPlanes, bmpXor.bmBitsPixel);
+               bmpXor.dsBm.bmWidth, bmpXor.dsBm.bmHeight, bmpXor.dsBm.bmWidthBytes,
+               bmpXor.dsBm.bmPlanes, bmpXor.dsBm.bmBitsPixel);
+        /* we can use either depth 1 or screen depth for xor bitmap */
+        if (bmpXor.dsBm.bmPlanes == 1 && bmpXor.dsBm.bmBitsPixel == 1) planes = bpp = 1;
+        sizeXor = bmpXor.dsBm.bmHeight * planes * get_bitmap_width_bytes( bmpXor.dsBm.bmWidth, bpp );
     }
     GetObjectW( iconinfo->hbmMask, sizeof(bmpAnd), &bmpAnd );
     TRACE("mask: width %d, height %d, width bytes %d, planes %u, bpp %u\n",
            bmpAnd.bmWidth, bmpAnd.bmHeight, bmpAnd.bmWidthBytes,
            bmpAnd.bmPlanes, bmpAnd.bmBitsPixel);
 
-    sizeXor = iconinfo->hbmColor ? (bmpXor.bmHeight * bmpXor.bmWidthBytes) : 0;
     sizeAnd = bmpAnd.bmHeight * get_bitmap_width_bytes(bmpAnd.bmWidth, 1);
 
     hObj = GlobalAlloc16( GMEM_MOVEABLE,
@@ -1837,11 +1860,11 @@ HICON WINAPI CreateIconIndirect(PICONINFO iconinfo)
 
         if (iconinfo->hbmColor)
         {
-            info->nWidth        = bmpXor.bmWidth;
-            info->nHeight       = bmpXor.bmHeight;
-            info->nWidthBytes   = bmpXor.bmWidthBytes;
-            info->bPlanes       = bmpXor.bmPlanes;
-            info->bBitsPerPixel = bmpXor.bmBitsPixel;
+            info->nWidth        = bmpXor.dsBm.bmWidth;
+            info->nHeight       = bmpXor.dsBm.bmHeight;
+            info->nWidthBytes   = bmpXor.dsBm.bmWidthBytes;
+            info->bPlanes       = planes;
+            info->bBitsPerPixel = bpp;
         }
         else
         {
@@ -1883,7 +1906,53 @@ HICON WINAPI CreateIconIndirect(PICONINFO iconinfo)
             GetBitmapBits( hbmp_mono, sizeAnd, (char*)(info + 1) );
             DeleteObject( hbmp_mono );
         }
-        if (iconinfo->hbmColor) GetBitmapBits( iconinfo->hbmColor, sizeXor, (char*)(info + 1) + sizeAnd );
+
+        if (iconinfo->hbmColor)
+        {
+            char *dst_bits = (char*)(info + 1) + sizeAnd;
+
+            if (bmpXor.dsBm.bmPlanes == planes && bmpXor.dsBm.bmBitsPixel == bpp)
+                GetBitmapBits( iconinfo->hbmColor, sizeXor, dst_bits );
+            else
+            {
+                BITMAPINFO bminfo;
+                int dib_width = get_dib_width_bytes( info->nWidth, info->bBitsPerPixel );
+                int bitmap_width = get_bitmap_width_bytes( info->nWidth, info->bBitsPerPixel );
+
+                bminfo.bmiHeader.biSize = sizeof(bminfo);
+                bminfo.bmiHeader.biWidth = info->nWidth;
+                bminfo.bmiHeader.biHeight = info->nHeight;
+                bminfo.bmiHeader.biPlanes = info->bPlanes;
+                bminfo.bmiHeader.biBitCount = info->bBitsPerPixel;
+                bminfo.bmiHeader.biCompression = BI_RGB;
+                bminfo.bmiHeader.biSizeImage = info->nHeight * dib_width;
+                bminfo.bmiHeader.biXPelsPerMeter = 0;
+                bminfo.bmiHeader.biYPelsPerMeter = 0;
+                bminfo.bmiHeader.biClrUsed = 0;
+                bminfo.bmiHeader.biClrImportant = 0;
+
+                /* swap lines for dib sections */
+                if (xor_objsize == sizeof(DIBSECTION))
+                    bminfo.bmiHeader.biHeight = -bminfo.bmiHeader.biHeight;
+
+                if (dib_width != bitmap_width)  /* need to fixup alignment */
+                {
+                    char *src_bits = HeapAlloc( GetProcessHeap(), 0, bminfo.bmiHeader.biSizeImage );
+
+                    if (src_bits && GetDIBits( screen_dc, iconinfo->hbmColor, 0, info->nHeight,
+                                               src_bits, &bminfo, DIB_RGB_COLORS ))
+                    {
+                        int y;
+                        for (y = 0; y < info->nHeight; y++)
+                            memcpy( dst_bits + y * bitmap_width, src_bits + y * dib_width, bitmap_width );
+                    }
+                    HeapFree( GetProcessHeap(), 0, src_bits );
+                }
+                else
+                    GetDIBits( screen_dc, iconinfo->hbmColor, 0, info->nHeight,
+                               dst_bits, &bminfo, DIB_RGB_COLORS );
+            }
+        }
         GlobalUnlock16( hObj );
     }
     return HICON_32(hObj);
@@ -2148,8 +2217,19 @@ static HBITMAP BITMAP_Load( HINSTANCE instance, LPCWSTR name,
     }
     else
     {
+        BITMAPFILEHEADER * bmfh;
+
         if (!(ptr = map_fileW( name, NULL ))) return 0;
         info = (BITMAPINFO *)(ptr + sizeof(BITMAPFILEHEADER));
+        bmfh = (BITMAPFILEHEADER *)ptr;
+        if (!(  bmfh->bfType == 0x4d42 /* 'BM' */ &&
+                bmfh->bfReserved1 == 0 &&
+                bmfh->bfReserved2 == 0))
+        {
+            WARN("Invalid/unsupported bitmap format!\n");
+            UnmapViewOfFile( ptr );
+            return 0;
+        }
     }
 
     size = bitmap_info_size(info, DIB_RGB_COLORS);
