@@ -843,17 +843,18 @@ LRESULT WINAPI DefWindowProcA( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam
         break;
 
     case WM_IME_KEYDOWN:
-        result = SendMessageA( hwnd, WM_KEYDOWN, wParam, lParam );
+        result = PostMessageA( hwnd, WM_KEYDOWN, wParam, lParam );
         break;
 
     case WM_IME_KEYUP:
-        result = SendMessageA( hwnd, WM_KEYUP, wParam, lParam );
+        result = PostMessageA( hwnd, WM_KEYUP, wParam, lParam );
         break;
 
     case WM_IME_STARTCOMPOSITION:
     case WM_IME_COMPOSITION:
     case WM_IME_ENDCOMPOSITION:
     case WM_IME_SELECT:
+    case WM_IME_NOTIFY:
         {
             HWND hwndIME;
 
@@ -992,6 +993,14 @@ LRESULT WINAPI DefWindowProcW(
         PostMessageW( hwnd, WM_CHAR, wParam, lParam );
         break;
 
+    case WM_IME_KEYDOWN:
+        result = PostMessageW( hwnd, WM_KEYDOWN, wParam, lParam );
+        break;
+
+    case WM_IME_KEYUP:
+        result = PostMessageW( hwnd, WM_KEYUP, wParam, lParam );
+        break;
+
     case WM_IME_SETCONTEXT:
         {
             HWND hwndIME;
@@ -1006,6 +1015,7 @@ LRESULT WINAPI DefWindowProcW(
     case WM_IME_COMPOSITION:
     case WM_IME_ENDCOMPOSITION:
     case WM_IME_SELECT:
+    case WM_IME_NOTIFY:
         {
             HWND hwndIME;
 
