@@ -314,7 +314,7 @@ static LRESULT call_hook_proc( HOOKPROC proc, INT id, INT code, WPARAM wparam, L
  *
  * Retrieve the hook procedure real value for a module-relative proc
  */
-static void *get_hook_proc( void *proc, const WCHAR *module )
+void *get_hook_proc( void *proc, const WCHAR *module )
 {
     HMODULE mod;
 
@@ -518,8 +518,6 @@ BOOL WINAPI UnhookWindowsHookEx( HHOOK hhook )
 {
     BOOL ret;
 
-    TRACE( "%p\n", hhook );
-
     SERVER_START_REQ( remove_hook )
     {
         req->handle = hhook;
@@ -712,8 +710,6 @@ BOOL WINAPI UnhookWinEvent(HWINEVENTHOOK hEventHook)
 {
     BOOL ret;
 
-    TRACE( "%p\n", hEventHook );
-
     SERVER_START_REQ( remove_hook )
     {
         req->handle = hEventHook;
