@@ -623,6 +623,8 @@ static HRGN send_ncpaint( HWND hwnd, HWND *child, UINT *flags )
 
     if (child) hwnd = *child;
 
+    if (hwnd == GetDesktopWindow()) return whole_rgn;
+
     if (whole_rgn)
     {
         RECT client, update;
@@ -859,20 +861,11 @@ static HWND fix_caret(HWND hWnd, const RECT *scroll_rect, INT dx, INT dy,
  */
 HDC WINAPI BeginPaint( HWND hwnd, PAINTSTRUCT *lps )
 {
-    HWND full_handle;
     HRGN hrgn;
     UINT flags = UPDATE_NONCLIENT | UPDATE_ERASE | UPDATE_PAINT | UPDATE_INTERNALPAINT | UPDATE_NOCHILDREN;
 
     if (!lps) return 0;
 
-    if (!(full_handle = WIN_IsCurrentThread( hwnd )))
-    {
-        if (IsWindow(hwnd))
-            FIXME( "window %p belongs to other thread\n", hwnd );
-        return 0;
-    }
-    hwnd = full_handle;
-
     HideCaret( hwnd );
 
     if (!(hrgn = send_ncpaint( hwnd, NULL, &flags ))) return 0;
@@ -1461,7 +1454,7 @@ INT WINAPI ScrollWindowEx( HWND hwnd, INT dx, INT dy,
                 CombineRgn( hrgnWinupd, hrgnWinupd, hrgnTemp, RGN_OR );
             RedrawWindow( hwnd, NULL, hrgnTemp, rdw_flags);
 
-           /* Catch the case where the scolling amount exceeds the size of the
+           /* Catch the case where the scrolling amount exceeds the size of the
             * original window. This generated a second update area that is the
             * location where the original scrolled content would end up.
             * This second region is not returned by the ScrollDC and sets
