@@ -260,7 +260,9 @@ static const char * const MessageTypeNames[SPY_MAX_MSGNUM + 1] =
     "BM_CLICK",                 /* 0x00f5 */
     "BM_GETIMAGE",              /* 0x00f6 */
     "BM_SETIMAGE",              /* 0x00f7 */
-    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+    NULL, NULL, NULL, NULL, NULL, NULL,
+    "WM_INPUT_DEVICE_CHANGE",   /* 0x00fe */
+    "WM_INPUT",                 /* 0x00ff */
 
     "WM_KEYDOWN",               /* 0x0100 */
     "WM_KEYUP",                 /* 0x0101 */
@@ -270,11 +272,11 @@ static const char * const MessageTypeNames[SPY_MAX_MSGNUM + 1] =
     "WM_SYSKEYUP",              /* 0x0105 */
     "WM_SYSCHAR",               /* 0x0106 */
     "WM_SYSDEADCHAR",           /* 0x0107 */
-    "WM_KEYLAST",               /* 0x0108 */
     NULL,
-    "WM_CONVERTREQUEST",
-    "WM_CONVERTRESULT",
-    "WM_INTERIM",
+    "WM_UNICHAR",               /* 0x0109 */
+    "WM_CONVERTREQUEST",        /* 0x010a */
+    "WM_CONVERTRESULT",         /* 0x010b */
+    "WM_INTERIM",               /* 0x010c */
     "WM_IME_STARTCOMPOSITION",  /* 0x010d */
     "WM_IME_ENDCOMPOSITION",    /* 0x010e */
     "WM_IME_COMPOSITION",       /* 0x010f */
@@ -450,7 +452,8 @@ static const char * const MessageTypeNames[SPY_MAX_MSGNUM + 1] =
     "WM_XBUTTONDOWN",           /* 0x020B */
     "WM_XBUTTONUP",             /* 0x020C */
     "WM_XBUTTONDBLCLK",         /* 0x020D */
-    NULL, NULL,
+    "WM_MOUSEHWHEEL",           /* 0x020E */
+    NULL,
 
     "WM_PARENTNOTIFY",          /* 0x0210 */
     "WM_ENTERMENULOOP",         /* 0x0211 */
@@ -589,17 +592,24 @@ static const char * const MessageTypeNames[SPY_MAX_MSGNUM + 1] =
     "WM_PALETTEISCHANGING",
     "WM_PALETTECHANGED",
     "WM_HOTKEY",                /* 0x0312 */
-          NULL, NULL, NULL, NULL,
+    NULL, NULL, NULL, NULL,
     "WM_PRINT",                 /* 0x0317 */
     "WM_PRINTCLIENT",           /* 0x0318 */
     "WM_APPCOMMAND",            /* 0x0319 */
     "WM_THEMECHANGED",          /* 0x031A */
-    NULL, NULL, NULL, NULL, NULL,
+    NULL, NULL,
+    "WM_CLIPBOARDUPDATE",       /* 0x031D */
+    "WM_DWMCOMPOSITIONCHANGED", /* 0x031E */
+    "WM_DWMNCRENDERINGCHANGED", /* 0x031F */
+
+    "WM_DWMCOLORIZATIONCOLORCHANGED", /* 0x0320 */
+    "WM_DWMWINDOWMAXIMIZEDCHANGE", /* 0x0321 */
+    NULL, NULL, NULL, NULL, NULL, NULL,
+    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
 
     NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
-    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
-    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
-    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+    NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+    "WM_GETTITLEBARINFOEX",     /* 0x033F */
 
     /* 0x0340 */
     NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
@@ -1925,7 +1935,7 @@ static const SPY_NOTIFY spnfy_array[] = {
     SPNFY(RBN_DELETINGBAND,      NMREBAR),
     SPNFY(RBN_DELETEDBAND,       NMREBAR),
     SPNFY(RBN_CHILDSIZE,         NMREBARCHILDSIZE),
-    /* IP Adderss     0U-860U  to  0U-879U  */
+    /* IP address     0U-860U  to  0U-879U  */
     SPNFY(IPN_FIELDCHANGED,      NMHDR),
     /* Status bar     0U-880U  to  0U-899U  */
     SPNFY(SBN_SIMPLEMODECHANGE,  NMHDR),
@@ -2311,10 +2321,8 @@ static void SPY_DumpStructure(const SPY_INSTANCE *sp_e, BOOL enter)
         case LVM_GETSUBITEMRECT:
             {
                 LPRECT rc = (LPRECT) sp_e->lParam;
-                if (rc) {
-                    TRACE("lParam rect (%d,%d)-(%d,%d)\n",
-                          rc->left, rc->top, rc->right, rc->bottom);
-                }
+                if (rc)
+                    TRACE("lParam rect (%s)\n", wine_dbgstr_rect(rc));
                 break;
             }
         case LVM_SETITEMPOSITION32:
@@ -2394,10 +2402,9 @@ static void SPY_DumpStructure(const SPY_INSTANCE *sp_e, BOOL enter)
                       lpdis->CtlType, lpdis->CtlID);
                 TRACE("itemID=0x%08x itemAction=0x%08x itemState=0x%08x\n",
                       lpdis->itemID, lpdis->itemAction, lpdis->itemState);
-                TRACE("hWnd=%p hDC=%p (%d,%d)-(%d,%d) itemData=0x%08lx\n",
-                      lpdis->hwndItem, lpdis->hDC, lpdis->rcItem.left,
-                      lpdis->rcItem.top, lpdis->rcItem.right,
-                      lpdis->rcItem.bottom, lpdis->itemData);
+                TRACE("hWnd=%p hDC=%p (%s) itemData=0x%08lx\n",
+                      lpdis->hwndItem, lpdis->hDC,
+                      wine_dbgstr_rect(&lpdis->rcItem), lpdis->itemData);
             }
             break;
         case WM_MEASUREITEM:
@@ -2453,8 +2460,7 @@ static void SPY_DumpStructure(const SPY_INSTANCE *sp_e, BOOL enter)
         case WM_NCCALCSIZE:
             {
                 RECT *rc = (RECT *)sp_e->lParam;
-                TRACE("Rect (%d,%d)-(%d,%d)\n",
-                      rc->left, rc->top, rc->right, rc->bottom);
+                TRACE("Rect (%s)\n", wine_dbgstr_rect(rc));
             }
             break;
         case WM_NOTIFY:
