@@ -20,15 +20,31 @@
 
 #include "config.h"
 
+#include <stdio.h>
+
 #include "dosexe.h"
+#include "winternl.h"
 #include "wine/debug.h"
 #include "wine/winbase16.h"
 
-#include "thread.h"
-
 WINE_DEFAULT_DEBUG_CHANNEL(int);
 WINE_DECLARE_DEBUG_CHANNEL(relay);
 
+#define BCD_TO_BIN(x) ((x&15) + (x>>4)*10)
+#define BIN_TO_BCD(x) ((x%10) + ((x/10)<<4))
+
+static void WINAPI DOSVM_Int11Handler(CONTEXT86*);
+static void WINAPI DOSVM_Int12Handler(CONTEXT86*);
+static void WINAPI DOSVM_Int17Handler(CONTEXT86*);
+static void WINAPI DOSVM_Int19Handler(CONTEXT86*);
+static void WINAPI DOSVM_Int1aHandler(CONTEXT86*);
+static void WINAPI DOSVM_Int20Handler(CONTEXT86*);
+static void WINAPI DOSVM_Int29Handler(CONTEXT86*);
+static void WINAPI DOSVM_Int2aHandler(CONTEXT86*);
+static void WINAPI DOSVM_Int41Handler(CONTEXT86*);
+static void WINAPI DOSVM_Int4bHandler(CONTEXT86*);
+static void WINAPI DOSVM_Int5cHandler(CONTEXT86*);
+static void WINAPI DOSVM_DefaultHandler(CONTEXT86*);
 
 static FARPROC16     DOSVM_Vectors16[256];
 static FARPROC48     DOSVM_Vectors48[256];
@@ -59,7 +75,8 @@ static const INTPROC DOSVM_VectorsBuiltin[] =
   /* 58 */ 0,                  0,                  0,                  0,
   /* 5C */ DOSVM_Int5cHandler, 0,                  0,                  0,
   /* 60 */ 0,                  0,                  0,                  0,
-  /* 64 */ 0,                  0,                  0,                  DOSVM_Int67Handler
+  /* 64 */ 0,                  0,                  0,                  DOSVM_Int67Handler,
+  /* 68 */ DOSVM_DefaultHandler
 };
 
 
@@ -161,7 +178,7 @@ static void DOSVM_IntProcRelay( CONTEXT86 *context, LPVOID data )
 static void DOSVM_PrepareIRQ( CONTEXT86 *context, BOOL isbuiltin )
 {
     /* Disable virtual interrupts. */
-    NtCurrentTeb()->dpmi_vif = 0;
+    get_vm86_teb_info()->dpmi_vif = 0;
 
     if (!isbuiltin)
     {
@@ -241,7 +258,7 @@ static void DOSVM_PushFlags( CONTEXT86 *context, BOOL islong, BOOL isstub )
  * Pushes interrupt frame to stack and changes instruction 
  * pointer to interrupt handler.
  */
-void WINAPI DOSVM_EmulateInterruptPM( CONTEXT86 *context, BYTE intnum ) 
+BOOL WINAPI DOSVM_EmulateInterruptPM( CONTEXT86 *context, BYTE intnum ) 
 {
     TRACE_(relay)("Call DOS int 0x%02x ret=%04x:%08x\n"
                   "  eax=%08x ebx=%08x ecx=%08x edx=%08x\n"
@@ -308,10 +325,18 @@ void WINAPI DOSVM_EmulateInterruptPM( CONTEXT86 *context, BYTE intnum )
                               DOSVM_IntProcRelay, 
                               DOSVM_GetBuiltinHandler(intnum) );
     }
+    else if (wine_ldt_is_system(context->SegCs))
+    {
+        INTPROC proc;
+        if (intnum >= sizeof(DOSVM_VectorsBuiltin)/sizeof(INTPROC)) return FALSE;
+        if (!(proc = DOSVM_VectorsBuiltin[intnum])) return FALSE;
+        proc( context );
+    }
     else
     {
         DOSVM_HardwareInterruptPM( context, intnum );
     }
+    return TRUE;
 }
 
 
@@ -686,3 +711,358 @@ void WINAPI DOSVM_CallBuiltinHandler( CONTEXT86 *context, BYTE intnum )
   INTPROC proc = DOSVM_GetBuiltinHandler( intnum );
   proc( context );
 }
+
+
+/**********************************************************************
+ *	    DOSVM_Int11Handler
+ *
+ * Handler for int 11h (get equipment list).
+ *
+ *
+ * Borrowed from Ralph Brown's interrupt lists:
+ *
+ *   bits 15-14: number of parallel devices
+ *   bit     13: [Conv] Internal modem
+ *   bit     12: reserved
+ *   bits 11- 9: number of serial devices
+ *   bit      8: reserved
+ *   bits  7- 6: number of diskette drives minus one
+ *   bits  5- 4: Initial video mode:
+ *                 00b = EGA,VGA,PGA
+ *                 01b = 40 x 25 color
+ *                 10b = 80 x 25 color
+ *                 11b = 80 x 25 mono
+ *   bit      3: reserved
+ *   bit      2: [PS] =1 if pointing device
+ *               [non-PS] reserved
+ *   bit      1: =1 if math co-processor
+ *   bit      0: =1 if diskette available for boot
+ *
+ *
+ * Currently the only of these bits correctly set are:
+ *
+ *   bits 15-14   } Added by William Owen Smith,
+ *   bits 11-9    } wos@dcs.warwick.ac.uk
+ *   bits 7-6
+ *   bit  2       (always set)  ( bit 2 = 4 )
+ *   bit  1       } Robert 'Admiral' Coeyman
+ *                  All *nix systems either have a math processor or
+ *		     emulate one.
+ */
+static void WINAPI DOSVM_Int11Handler( CONTEXT86 *context )
+{
+    int diskdrives = 0;
+    int parallelports = 0;
+    int serialports = 0;
+    int x;
+
+    if (GetDriveTypeA("A:\\") == DRIVE_REMOVABLE) diskdrives++;
+    if (GetDriveTypeA("B:\\") == DRIVE_REMOVABLE) diskdrives++;
+    if (diskdrives) diskdrives--;
+
+    for (x=0; x < 9; x++)
+    {
+        HANDLE handle;
+        char file[10];
+
+        /* serial port name */
+        sprintf( file, "\\\\.\\COM%d", x+1 );
+        handle = CreateFileA( file, 0, FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_EXISTING, 0, 0 );
+        if (handle != INVALID_HANDLE_VALUE)
+        {
+            CloseHandle( handle );
+            serialports++;
+        }
+
+        sprintf( file, "\\\\.\\LPT%d", x+1 );
+        handle = CreateFileA( file, 0, FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_EXISTING, 0, 0 );
+        if (handle != INVALID_HANDLE_VALUE)
+        {
+            CloseHandle( handle );
+            parallelports++;
+        }
+    }
+
+    if (serialports > 7) /* 3 bits -- maximum value = 7 */
+        serialports = 7;
+
+    if (parallelports > 3) /* 2 bits -- maximum value = 3 */
+        parallelports = 3;
+
+    SET_AX( context,
+            (diskdrives << 6) | (serialports << 9) | (parallelports << 14) | 0x06 );
+}
+
+
+/**********************************************************************
+ *         DOSVM_Int12Handler
+ *
+ * Handler for int 12h (get memory size).
+ */
+static void WINAPI DOSVM_Int12Handler( CONTEXT86 *context )
+{
+    SET_AX( context, 640 );
+}
+
+
+/**********************************************************************
+ *          DOSVM_Int17Handler
+ *
+ * Handler for int 17h (printer - output character).
+ */
+static void WINAPI DOSVM_Int17Handler( CONTEXT86 *context )
+{
+    switch( AH_reg(context) )
+    {
+       case 0x00:/* Send character*/
+            FIXME("Send character not supported yet\n");
+            SET_AH( context, 0x00 );/*Timeout*/
+            break;
+        case 0x01:              /* PRINTER - INITIALIZE */
+            FIXME("Initialize Printer - Not Supported\n");
+            SET_AH( context, 0x30 ); /* selected | out of paper */
+            break;
+        case 0x02:              /* PRINTER - GET STATUS */
+            FIXME("Get Printer Status - Not Supported\n");
+            break;
+        default:
+            SET_AH( context, 0 ); /* time out */
+            INT_BARF( context, 0x17 );
+    }
+}
+
+
+/**********************************************************************
+ *          DOSVM_Int19Handler
+ *
+ * Handler for int 19h (Reboot).
+ */
+static void WINAPI DOSVM_Int19Handler( CONTEXT86 *context )
+{
+    TRACE( "Attempted Reboot\n" );
+    ExitProcess(0);
+}
+
+
+/**********************************************************************
+ *         DOSVM_Int1aHandler
+ *
+ * Handler for int 1ah.
+ */
+static void WINAPI DOSVM_Int1aHandler( CONTEXT86 *context )
+{
+    switch(AH_reg(context))
+    {
+    case 0x00: /* GET SYSTEM TIME */
+        {
+            BIOSDATA *data = DOSVM_BiosData();
+            SET_CX( context, HIWORD(data->Ticks) );
+            SET_DX( context, LOWORD(data->Ticks) );
+            SET_AL( context, 0 ); /* FIXME: midnight flag is unsupported */
+            TRACE( "GET SYSTEM TIME - ticks=%d\n", data->Ticks );
+        }
+        break;
+
+    case 0x01: /* SET SYSTEM TIME */
+        FIXME( "SET SYSTEM TIME - not allowed\n" );
+        break;
+
+    case 0x02: /* GET REAL-TIME CLOCK TIME */
+        TRACE( "GET REAL-TIME CLOCK TIME\n" );
+        {
+            SYSTEMTIME systime;
+            GetLocalTime( &systime );
+            SET_CH( context, BIN_TO_BCD(systime.wHour) );
+            SET_CL( context, BIN_TO_BCD(systime.wMinute) );
+            SET_DH( context, BIN_TO_BCD(systime.wSecond) );
+            SET_DL( context, 0 ); /* FIXME: assume no daylight saving */
+            RESET_CFLAG(context);
+        }
+        break;
+
+    case 0x03: /* SET REAL-TIME CLOCK TIME */
+        FIXME( "SET REAL-TIME CLOCK TIME - not allowed\n" );
+        break;
+
+    case 0x04: /* GET REAL-TIME CLOCK DATE */
+        TRACE( "GET REAL-TIME CLOCK DATE\n" );
+        {
+            SYSTEMTIME systime;
+            GetLocalTime( &systime );
+            SET_CH( context, BIN_TO_BCD(systime.wYear / 100) );
+            SET_CL( context, BIN_TO_BCD(systime.wYear % 100) );
+            SET_DH( context, BIN_TO_BCD(systime.wMonth) );
+            SET_DL( context, BIN_TO_BCD(systime.wDay) );
+            RESET_CFLAG(context);
+        }
+        break;
+
+    case 0x05: /* SET REAL-TIME CLOCK DATE */
+        FIXME( "SET REAL-TIME CLOCK DATE - not allowed\n" );
+        break;
+
+    case 0x06: /* SET ALARM */
+        FIXME( "SET ALARM - unimplemented\n" );
+        break;
+
+    case 0x07: /* CANCEL ALARM */
+        FIXME( "CANCEL ALARM - unimplemented\n" );
+        break;
+
+    case 0x08: /* SET RTC ACTIVATED POWER ON MODE */
+    case 0x09: /* READ RTC ALARM TIME AND STATUS */
+    case 0x0a: /* READ SYSTEM-TIMER DAY COUNTER */
+    case 0x0b: /* SET SYSTEM-TIMER DAY COUNTER */
+    case 0x0c: /* SET RTC DATE/TIME ACTIVATED POWER-ON MODE */
+    case 0x0d: /* RESET RTC DATE/TIME ACTIVATED POWER-ON MODE */
+    case 0x0e: /* GET RTC DATE/TIME ALARM AND STATUS */
+    case 0x0f: /* INITIALIZE REAL-TIME CLOCK */
+        INT_BARF( context, 0x1a );
+        break;
+
+    case 0xb0:
+        if (CX_reg(context) == 0x4d52 &&
+            DX_reg(context) == 0x4349 &&
+            AL_reg(context) == 0x01)
+        {
+            /*
+             * Microsoft Real-Time Compression Interface (MRCI).
+             * Ignoring this call indicates MRCI is not supported.
+             */
+            TRACE( "Microsoft Real-Time Compression Interface - not supported\n" );
+        }
+        else
+        {
+            INT_BARF(context, 0x1a);
+        }
+        break;
+
+    default:
+        INT_BARF( context, 0x1a );
+    }
+}
+
+
+/**********************************************************************
+ *	    DOSVM_Int20Handler
+ *
+ * Handler for int 20h.
+ */
+static void WINAPI DOSVM_Int20Handler( CONTEXT86 *context )
+{
+    if (DOSVM_IsWin16())
+        ExitThread( 0 );
+    else if(ISV86(context))
+        MZ_Exit( context, TRUE, 0 );
+    else
+        ERR( "Called from DOS protected mode\n" );
+}
+
+
+/**********************************************************************
+ *	    DOSVM_Int29Handler
+ *
+ * Handler for int 29h (fast console output)
+ */
+static void WINAPI DOSVM_Int29Handler( CONTEXT86 *context )
+{
+   /* Yes, it seems that this is really all this interrupt does. */
+   DOSVM_PutChar(AL_reg(context));
+}
+
+
+/**********************************************************************
+ *         DOSVM_Int2aHandler
+ *
+ * Handler for int 2ah (network).
+ */
+static void WINAPI DOSVM_Int2aHandler( CONTEXT86 *context )
+{
+    switch(AH_reg(context))
+    {
+    case 0x00:                             /* NETWORK INSTALLATION CHECK */
+        break;
+
+    default:
+       INT_BARF( context, 0x2a );
+    }
+}
+
+
+/***********************************************************************
+ *           DOSVM_Int41Handler
+ */
+static void WINAPI DOSVM_Int41Handler( CONTEXT86 *context )
+{
+    if ( ISV86(context) )
+    {
+        /* Real-mode debugger services */
+        switch ( AX_reg(context) )
+        {
+        default:
+            INT_BARF( context, 0x41 );
+            break;
+        }
+    }
+    else
+    {
+        /* Protected-mode debugger services */
+        switch ( AX_reg(context) )
+        {
+        case 0x4f:
+        case 0x50:
+        case 0x150:
+        case 0x51:
+        case 0x52:
+        case 0x152:
+        case 0x59:
+        case 0x5a:
+        case 0x5b:
+        case 0x5c:
+        case 0x5d:
+            /* Notifies the debugger of a lot of stuff. We simply ignore it
+               for now, but some of the info might actually be useful ... */
+            break;
+
+        default:
+            INT_BARF( context, 0x41 );
+            break;
+        }
+    }
+}
+
+
+/***********************************************************************
+ *           DOSVM_Int4bHandler
+ *
+ */
+static void WINAPI DOSVM_Int4bHandler( CONTEXT86 *context )
+{
+    switch(AH_reg(context))
+    {
+    case 0x81: /* Virtual DMA Spec (IBM SCSI interface) */
+        if(AL_reg(context) != 0x02) /* if not install check */
+        {
+            SET_CFLAG(context);
+            SET_AL( context, 0x0f ); /* function is not implemented */
+        }
+        break;
+    default:
+        INT_BARF(context, 0x4b);
+    }
+}
+
+
+/***********************************************************************
+ *           DOSVM_Int5cHandler
+ *
+ * Called from NetBIOSCall16.
+ */
+static void WINAPI DOSVM_Int5cHandler( CONTEXT86 *context )
+{
+    BYTE* ptr;
+    ptr = MapSL( MAKESEGPTR(context->SegEs,BX_reg(context)) );
+    FIXME("(%p): command code %02x (ignored)\n",context, *ptr);
+    *(ptr+0x01) = 0xFB; /* NetBIOS emulator not found */
+    SET_AL( context, 0xFB );
+}
