@@ -26,6 +26,12 @@
 extern "C" {
 #endif /* defined(__cplusplus) */
 
+#ifdef _USER32_
+#define WINUSERAPI
+#else
+#define WINUSERAPI DECLSPEC_IMPORT
+#endif
+
 /* Codepage Constants
  */
 
@@ -192,6 +198,7 @@ extern "C" {
 #define XTYP_DISCONNECT		(0x00C0 | XCLASS_NOTIFICATION | XTYPF_NOBLOCK )
 #define XTYP_UNREGISTER		(0x00D0 | XCLASS_NOTIFICATION | XTYPF_NOBLOCK )
 #define XTYP_WILDCONNECT	(0x00E0 | XCLASS_DATA | XTYPF_NOBLOCK)
+#define XTYP_MONITOR		(0x00F0 | XCLASS_NOTIFICATION | XTYPF_NOBLOCK)
 
 #define XTYP_MASK		0x00F0
 #define XTYP_SHIFT		4
@@ -333,42 +340,40 @@ typedef struct tagCONVINFO
 
 /*            Interface Definitions		*/
 
-
-UINT    WINAPI DdeInitializeA(LPDWORD,PFNCALLBACK,DWORD,DWORD);
-UINT    WINAPI DdeInitializeW(LPDWORD,PFNCALLBACK,DWORD,DWORD);
-#define   DdeInitialize WINELIB_NAME_AW(DdeInitialize)
-BOOL    WINAPI DdeUninitialize(DWORD);
-HCONVLIST WINAPI DdeConnectList(DWORD,HSZ,HSZ,HCONVLIST,PCONVCONTEXT);
-HCONV     WINAPI DdeQueryNextServer(HCONVLIST, HCONV);
-DWORD     WINAPI DdeQueryStringA(DWORD, HSZ, LPSTR, DWORD, INT);
-DWORD     WINAPI DdeQueryStringW(DWORD, HSZ, LPWSTR, DWORD, INT);
-#define   DdeQueryString WINELIB_NAME_AW(DdeQueryString)
-BOOL      WINAPI DdeDisconnectList(HCONVLIST);
-HCONV     WINAPI DdeConnect(DWORD,HSZ,HSZ,PCONVCONTEXT);
-BOOL      WINAPI DdeDisconnect(HCONV);
-HDDEDATA  WINAPI DdeCreateDataHandle(DWORD,LPBYTE,DWORD,DWORD,HSZ,UINT,UINT);
-HCONV     WINAPI DdeReconnect(HCONV);
-HSZ       WINAPI DdeCreateStringHandleA(DWORD,LPCSTR,INT);
-HSZ       WINAPI DdeCreateStringHandleW(DWORD,LPCWSTR,INT);
-#define   DdeCreateStringHandle WINELIB_NAME_AW(DdeCreateStringHandle)
-BOOL      WINAPI DdeFreeStringHandle(DWORD,HSZ);
-BOOL      WINAPI DdeFreeDataHandle(HDDEDATA);
-BOOL      WINAPI DdeKeepStringHandle(DWORD,HSZ);
-HDDEDATA  WINAPI DdeClientTransaction(LPBYTE,DWORD,HCONV,HSZ,UINT,UINT,DWORD,LPDWORD);
-BOOL	  WINAPI DdeAbandonTransaction(DWORD idInst, HCONV hConv, DWORD idTransaction);
-BOOL      WINAPI DdeImpersonateClient(HCONV);
-BOOL      WINAPI DdePostAdvise(DWORD,HSZ,HSZ);
-HDDEDATA  WINAPI DdeAddData(HDDEDATA,LPBYTE,DWORD,DWORD);
-DWORD     WINAPI DdeGetData(HDDEDATA,LPBYTE,DWORD,DWORD);
-LPBYTE    WINAPI DdeAccessData(HDDEDATA,LPDWORD);
-BOOL      WINAPI DdeUnaccessData(HDDEDATA);
-BOOL      WINAPI DdeEnableCallback(DWORD,HCONV,UINT);
-INT       WINAPI DdeCmpStringHandles(HSZ,HSZ);
-BOOL      WINAPI DdeSetUserHandle(HCONV,DWORD,DWORD);
-
-HDDEDATA  WINAPI DdeNameService(DWORD,HSZ,HSZ,UINT);
-UINT      WINAPI DdeGetLastError(DWORD);
-UINT      WINAPI DdeQueryConvInfo(HCONV,DWORD,PCONVINFO);
+WINUSERAPI BOOL      WINAPI DdeAbandonTransaction(DWORD idInst, HCONV hConv, DWORD idTransaction);
+WINUSERAPI LPBYTE    WINAPI DdeAccessData(HDDEDATA,LPDWORD);
+WINUSERAPI HDDEDATA  WINAPI DdeAddData(HDDEDATA,LPBYTE,DWORD,DWORD);
+WINUSERAPI HDDEDATA  WINAPI DdeClientTransaction(LPBYTE,DWORD,HCONV,HSZ,UINT,UINT,DWORD,LPDWORD);
+WINUSERAPI INT       WINAPI DdeCmpStringHandles(HSZ,HSZ);
+WINUSERAPI HCONV     WINAPI DdeConnect(DWORD,HSZ,HSZ,PCONVCONTEXT);
+WINUSERAPI HCONVLIST WINAPI DdeConnectList(DWORD,HSZ,HSZ,HCONVLIST,PCONVCONTEXT);
+WINUSERAPI HDDEDATA  WINAPI DdeCreateDataHandle(DWORD,LPBYTE,DWORD,DWORD,HSZ,UINT,UINT);
+WINUSERAPI HSZ       WINAPI DdeCreateStringHandleA(DWORD,LPCSTR,INT);
+WINUSERAPI HSZ       WINAPI DdeCreateStringHandleW(DWORD,LPCWSTR,INT);
+#define                     DdeCreateStringHandle WINELIB_NAME_AW(DdeCreateStringHandle)
+WINUSERAPI BOOL      WINAPI DdeDisconnect(HCONV);
+WINUSERAPI BOOL      WINAPI DdeDisconnectList(HCONVLIST);
+WINUSERAPI BOOL      WINAPI DdeEnableCallback(DWORD,HCONV,UINT);
+WINUSERAPI BOOL      WINAPI DdeFreeDataHandle(HDDEDATA);
+WINUSERAPI BOOL      WINAPI DdeFreeStringHandle(DWORD,HSZ);
+WINUSERAPI DWORD     WINAPI DdeGetData(HDDEDATA,LPBYTE,DWORD,DWORD);
+WINUSERAPI UINT      WINAPI DdeGetLastError(DWORD);
+WINUSERAPI BOOL      WINAPI DdeImpersonateClient(HCONV);
+WINUSERAPI UINT      WINAPI DdeInitializeA(LPDWORD,PFNCALLBACK,DWORD,DWORD);
+WINUSERAPI UINT      WINAPI DdeInitializeW(LPDWORD,PFNCALLBACK,DWORD,DWORD);
+#define                     DdeInitialize WINELIB_NAME_AW(DdeInitialize)
+WINUSERAPI BOOL      WINAPI DdeKeepStringHandle(DWORD,HSZ);
+WINUSERAPI HDDEDATA  WINAPI DdeNameService(DWORD,HSZ,HSZ,UINT);
+WINUSERAPI BOOL      WINAPI DdePostAdvise(DWORD,HSZ,HSZ);
+WINUSERAPI UINT      WINAPI DdeQueryConvInfo(HCONV,DWORD,PCONVINFO);
+WINUSERAPI HCONV     WINAPI DdeQueryNextServer(HCONVLIST, HCONV);
+WINUSERAPI DWORD     WINAPI DdeQueryStringA(DWORD, HSZ, LPSTR, DWORD, INT);
+WINUSERAPI DWORD     WINAPI DdeQueryStringW(DWORD, HSZ, LPWSTR, DWORD, INT);
+#define                     DdeQueryString WINELIB_NAME_AW(DdeQueryString)
+WINUSERAPI HCONV     WINAPI DdeReconnect(HCONV);
+WINUSERAPI BOOL      WINAPI DdeSetUserHandle(HCONV,DWORD,DWORD);
+WINUSERAPI BOOL      WINAPI DdeUnaccessData(HDDEDATA);
+WINUSERAPI BOOL      WINAPI DdeUninitialize(DWORD);
 
 #ifdef __cplusplus
 } /* extern "C" */
