@@ -38,6 +38,8 @@ class GpLineGradient : public GpBrush {};
 class GpTexture : public GpBrush {};
 class GpFont {};
 class GpStringFormat {};
+class GpRegion {};
+class CGpEffect {};
 
 #else /* end of c++ declarations */
 
@@ -58,6 +60,8 @@ typedef struct GpLineGradient GpLineGradient;
 typedef struct GpTexture GpTexture;
 typedef struct GpFont GpFont;
 typedef struct GpStringFormat GpStringFormat;
+typedef struct GpRegion GpRegion;
+typedef struct CGpEffect CGpEffect;
 
 #endif /* end of c declarations */
 
@@ -76,5 +80,6 @@ typedef DashStyle GpDashStyle;
 typedef MatrixOrder GpMatrixOrder;
 typedef Point GpPoint;
 typedef WrapMode GpWrapMode;
+typedef Color GpColor;
 
 #endif
