@@ -86,6 +86,12 @@ void        free(void*);
 void*       malloc(size_t);
 void*       realloc(void*,size_t);
 
+void        _aligned_free(void*);
+void*       _aligned_malloc(size_t,size_t);
+void*       _aligned_offset_malloc(size_t,size_t,size_t);
+void*       _aligned_realloc(void*,size_t,size_t);
+void*       _aligned_offset_realloc(void*,size_t,size_t,size_t);
+
 size_t _get_sbh_threshold(void);
 int _set_sbh_threshold(size_t size);
 
