@@ -372,7 +372,7 @@ DWORD svcctl_CreateServiceW(
     entry->config.lpServiceStartName = strdupW(lpServiceStartName);
     entry->config.lpDisplayName = strdupW(lpDisplayName);
 
-    if (lpdwTagId)      /* TODO: in most situations a non-NULL tagid will generate a ERROR_INVALID_PARAMETER */
+    if (lpdwTagId)      /* TODO: In most situations a non-NULL TagId will generate an ERROR_INVALID_PARAMETER. */
         entry->config.dwTagId = *lpdwTagId;
     else
         entry->config.dwTagId = 0;
@@ -479,7 +479,7 @@ DWORD svcctl_ChangeServiceConfigW(
         DWORD dwPasswordSize,
         LPCWSTR lpDisplayName)
 {
-    struct service_entry new_entry;
+    struct service_entry new_entry, *entry;
     struct sc_service_handle *service;
     DWORD err;
 
@@ -501,7 +501,8 @@ DWORD svcctl_ChangeServiceConfigW(
     }
 
     if (lpDisplayName != NULL &&
-        scmdatabase_find_service_by_displayname(service->service_entry->db, lpDisplayName))
+        (entry = scmdatabase_find_service_by_displayname(service->service_entry->db, lpDisplayName)) &&
+        (entry != service->service_entry))
     {
         service_unlock(service->service_entry);
         return ERROR_DUPLICATE_SERVICE_NAME;
@@ -541,7 +542,7 @@ DWORD svcctl_ChangeServiceConfigW(
 
     if (!validate_service_config(&new_entry))
     {
-        WINE_ERR("The configuration after the change wouldn't be valid");
+        WINE_ERR("The configuration after the change wouldn't be valid\n");
         service_unlock(service->service_entry);
         return ERROR_INVALID_PARAMETER;
     }
@@ -654,187 +655,6 @@ DWORD svcctl_QueryServiceStatusEx(
     return ERROR_SUCCESS;
 }
 
-/* only one service started at a time, so there is no race on the registry
- * value here */
-static LPWSTR service_get_pipe_name(void)
-{
-    static const WCHAR format[] = { '\\','\\','.','\\','p','i','p','e','\\',
-        'n','e','t','\\','N','t','C','o','n','t','r','o','l','P','i','p','e','%','u',0};
-    static const WCHAR service_current_key_str[] = { 'S','Y','S','T','E','M','\\',
-        'C','u','r','r','e','n','t','C','o','n','t','r','o','l','S','e','t','\\',
-        'C','o','n','t','r','o','l','\\',
-        'S','e','r','v','i','c','e','C','u','r','r','e','n','t',0};
-    LPWSTR name;
-    DWORD len;
-    HKEY service_current_key;
-    DWORD service_current = -1;
-    LONG ret;
-    DWORD type;
-
-    ret = RegCreateKeyExW(HKEY_LOCAL_MACHINE, service_current_key_str, 0,
-        NULL, REG_OPTION_VOLATILE, KEY_SET_VALUE | KEY_QUERY_VALUE, NULL,
-        &service_current_key, NULL);
-    if (ret != ERROR_SUCCESS)
-        return NULL;
-    len = sizeof(service_current);
-    ret = RegQueryValueExW(service_current_key, NULL, NULL, &type,
-        (BYTE *)&service_current, &len);
-    if ((ret == ERROR_SUCCESS && type == REG_DWORD) || ret == ERROR_FILE_NOT_FOUND)
-    {
-        service_current++;
-        RegSetValueExW(service_current_key, NULL, 0, REG_DWORD,
-            (BYTE *)&service_current, sizeof(service_current));
-    }
-    RegCloseKey(service_current_key);
-    if ((ret != ERROR_SUCCESS || type != REG_DWORD) && (ret != ERROR_FILE_NOT_FOUND))
-        return NULL;
-    len = sizeof(format)/sizeof(WCHAR) + 10 /* strlenW("4294967295") */;
-    name = HeapAlloc(GetProcessHeap(), 0, len * sizeof(WCHAR));
-    if (!name)
-        return NULL;
-    snprintfW(name, len, format, service_current);
-    return name;
-}
-
-static DWORD service_start_process(struct service_entry *service_entry, HANDLE *process)
-{
-    PROCESS_INFORMATION pi;
-    STARTUPINFOW si;
-    LPWSTR path = NULL;
-    DWORD size;
-    BOOL r;
-
-    service_lock_exclusive(service_entry);
-
-    if (service_entry->config.dwServiceType == SERVICE_KERNEL_DRIVER)
-    {
-        static const WCHAR winedeviceW[] = {'\\','w','i','n','e','d','e','v','i','c','e','.','e','x','e',' ',0};
-        DWORD len = GetSystemDirectoryW( NULL, 0 ) + sizeof(winedeviceW)/sizeof(WCHAR) + strlenW(service_entry->name);
-
-        if (!(path = HeapAlloc( GetProcessHeap(), 0, len * sizeof(WCHAR) )))
-            return ERROR_NOT_ENOUGH_SERVER_MEMORY;
-        GetSystemDirectoryW( path, len );
-        lstrcatW( path, winedeviceW );
-        lstrcatW( path, service_entry->name );
-    }
-    else
-    {
-        size = ExpandEnvironmentStringsW(service_entry->config.lpBinaryPathName,NULL,0);
-        path = HeapAlloc(GetProcessHeap(),0,size*sizeof(WCHAR));
-        if (!path)
-            return ERROR_NOT_ENOUGH_SERVER_MEMORY;
-        ExpandEnvironmentStringsW(service_entry->config.lpBinaryPathName,path,size);
-    }
-
-    ZeroMemory(&si, sizeof(STARTUPINFOW));
-    si.cb = sizeof(STARTUPINFOW);
-    if (!(service_entry->config.dwServiceType & SERVICE_INTERACTIVE_PROCESS))
-    {
-        static WCHAR desktopW[] = {'_','_','w','i','n','e','s','e','r','v','i','c','e','_','w','i','n','s','t','a','t','i','o','n','\\','D','e','f','a','u','l','t',0};
-        si.lpDesktop = desktopW;
-    }
-
-    service_entry->status.dwCurrentState = SERVICE_START_PENDING;
-    service_entry->status.dwProcessId = pi.dwProcessId;
-
-    service_unlock(service_entry);
-
-    r = CreateProcessW(NULL, path, NULL, NULL, FALSE, 0, NULL, NULL, &si, &pi);
-    HeapFree(GetProcessHeap(),0,path);
-    if (!r)
-    {
-        service_lock_exclusive(service_entry);
-        service_entry->status.dwCurrentState = SERVICE_STOPPED;
-        service_entry->status.dwProcessId = 0;
-        service_unlock(service_entry);
-        return GetLastError();
-    }
-
-    *process = pi.hProcess;
-    CloseHandle( pi.hThread );
-
-    return ERROR_SUCCESS;
-}
-
-static DWORD service_wait_for_startup(struct service_entry *service_entry, HANDLE process_handle)
-{
-    WINE_TRACE("%p\n", service_entry);
-
-    for (;;)
-    {
-        DWORD dwCurrentStatus;
-        HANDLE handles[2] = { service_entry->status_changed_event, process_handle };
-        DWORD ret;
-        ret = WaitForMultipleObjects(sizeof(handles)/sizeof(handles[0]), handles, FALSE, 20000);
-        if (ret != WAIT_OBJECT_0)
-            return ERROR_SERVICE_REQUEST_TIMEOUT;
-        service_lock_shared(service_entry);
-        dwCurrentStatus = service_entry->status.dwCurrentState;
-        service_unlock(service_entry);
-        if (dwCurrentStatus == SERVICE_RUNNING)
-        {
-            WINE_TRACE("Service started successfully\n");
-            return ERROR_SUCCESS;
-        }
-        if (dwCurrentStatus != SERVICE_START_PENDING)
-            return ERROR_SERVICE_REQUEST_TIMEOUT;
-    }
-}
-
-/******************************************************************************
- * service_send_start_message
- */
-static BOOL service_send_start_message(HANDLE pipe, LPCWSTR *argv, DWORD argc)
-{
-    DWORD i, len, count, result;
-    service_start_info *ssi;
-    LPWSTR p;
-    BOOL r;
-
-    WINE_TRACE("%p %p %d\n", pipe, argv, argc);
-
-    /* FIXME: this can block so should be done in another thread */
-    r = ConnectNamedPipe(pipe, NULL);
-    if (!r && GetLastError() != ERROR_PIPE_CONNECTED)
-    {
-        WINE_ERR("pipe connect failed\n");
-        return FALSE;
-    }
-
-    /* calculate how much space do we need to send the startup info */
-    len = 1;
-    for (i=0; i<argc; i++)
-        len += strlenW(argv[i])+1;
-
-    ssi = HeapAlloc(GetProcessHeap(),0,FIELD_OFFSET(service_start_info, str[len]));
-    ssi->cmd = WINESERV_STARTINFO;
-    ssi->size = len;
-
-    /* copy service args into a single buffer*/
-    p = &ssi->str[0];
-    for (i=0; i<argc; i++)
-    {
-        strcpyW(p, argv[i]);
-        p += strlenW(p) + 1;
-    }
-    *p=0;
-
-    r = WriteFile(pipe, ssi, FIELD_OFFSET(service_start_info, str[len]), &count, NULL);
-    if (r)
-    {
-        r = ReadFile(pipe, &result, sizeof result, &count, NULL);
-        if (r && result)
-        {
-            SetLastError(result);
-            r = FALSE;
-        }
-    }
-
-    HeapFree(GetProcessHeap(),0,ssi);
-
-    return r;
-}
-
 /******************************************************************************
  * service_accepts_control
  */
@@ -888,15 +708,24 @@ static BOOL service_accepts_control(const struct service_entry *service, DWORD d
 /******************************************************************************
  * service_send_control
  */
-static BOOL service_send_control(HANDLE pipe, DWORD dwControl, DWORD *result)
+static BOOL service_send_control(struct service_entry *service, HANDLE pipe, DWORD dwControl, DWORD *result)
 {
-    DWORD cmd[2], count = 0;
+    service_start_info *ssi;
+    DWORD len, count = 0;
     BOOL r;
 
-    cmd[0] = WINESERV_SENDCONTROL;
-    cmd[1] = dwControl;
-    r = WriteFile(pipe, cmd, sizeof cmd, &count, NULL);
-    if (!r || count != sizeof cmd)
+    /* calculate how much space we need to send the startup info */
+    len = strlenW(service->name) + 1;
+
+    ssi = HeapAlloc(GetProcessHeap(),0,FIELD_OFFSET(service_start_info, data[len]));
+    ssi->cmd = WINESERV_SENDCONTROL;
+    ssi->control = dwControl;
+    ssi->total_size = FIELD_OFFSET(service_start_info, data[len]);
+    ssi->name_size = strlenW(service->name) + 1;
+    strcpyW( ssi->data, service->name );
+
+    r = WriteFile(pipe, ssi, ssi->total_size, &count, NULL);
+    if (!r || count != ssi->total_size)
     {
         WINE_ERR("service protocol error - failed to write pipe!\n");
         return r;
@@ -915,60 +744,13 @@ DWORD svcctl_StartServiceW(
 {
     struct sc_service_handle *service;
     DWORD err;
-    LPWSTR name;
-    HANDLE process_handle = NULL;
 
     WINE_TRACE("(%p, %d, %p)\n", hService, dwNumServiceArgs, lpServiceArgVectors);
 
     if ((err = validate_service_handle(hService, SERVICE_START, &service)) != 0)
         return err;
 
-    err = scmdatabase_lock_startup(service->service_entry->db);
-    if (err != ERROR_SUCCESS)
-        return err;
-
-    if (service->service_entry->control_pipe != INVALID_HANDLE_VALUE)
-    {
-        scmdatabase_unlock_startup(service->service_entry->db);
-        return ERROR_SERVICE_ALREADY_RUNNING;
-    }
-
-    service->service_entry->control_mutex = CreateMutexW(NULL, TRUE, NULL);
-
-    if (!service->service_entry->status_changed_event)
-        service->service_entry->status_changed_event = CreateEventW(NULL, TRUE, FALSE, NULL);
-
-    name = service_get_pipe_name();
-    service->service_entry->control_pipe = CreateNamedPipeW(name, PIPE_ACCESS_DUPLEX,
-                  PIPE_TYPE_BYTE|PIPE_WAIT, 1, 256, 256, 10000, NULL );
-    HeapFree(GetProcessHeap(), 0, name);
-    if (service->service_entry->control_pipe==INVALID_HANDLE_VALUE)
-    {
-        WINE_ERR("failed to create pipe for %s, error = %d\n",
-            wine_dbgstr_w(service->service_entry->name), GetLastError());
-        scmdatabase_unlock_startup(service->service_entry->db);
-        return GetLastError();
-    }
-
-    err = service_start_process(service->service_entry, &process_handle);
-
-    if (err == ERROR_SUCCESS)
-    {
-        if (!service_send_start_message(service->service_entry->control_pipe,
-                lpServiceArgVectors, dwNumServiceArgs))
-            err = ERROR_SERVICE_REQUEST_TIMEOUT;
-    }
-
-    WINE_TRACE("returning %d\n", err);
-
-    if (err == ERROR_SUCCESS)
-        err = service_wait_for_startup(service->service_entry, process_handle);
-
-    if (process_handle)
-        CloseHandle(process_handle);
-
-    ReleaseMutex(service->service_entry->control_mutex);
-    scmdatabase_unlock_startup(service->service_entry->db);
+    err = service_start(service->service_entry, dwNumServiceArgs, lpServiceArgVectors);
 
     return err;
 }
@@ -1054,17 +836,17 @@ DWORD svcctl_ControlService(
     if (dwControl == SERVICE_CONTROL_STOP)
     {
         service->service_entry->control_mutex = NULL;
-        service->service_entry->control_pipe = NULL;
+        service->service_entry->control_pipe = INVALID_HANDLE_VALUE;
     }
 
     service_unlock(service->service_entry);
 
     ret = WaitForSingleObject(control_mutex, 30000);
-    if (ret)
+    if (ret == WAIT_OBJECT_0)
     {
         DWORD result = ERROR_SUCCESS;
 
-        ret = service_send_control(control_pipe, dwControl, &result);
+        ret = service_send_control(service->service_entry, control_pipe, dwControl, &result);
 
         if (dwControl == SERVICE_CONTROL_STOP)
         {
@@ -1333,11 +1115,10 @@ DWORD svcctl_QueryServiceConfig2W(
 }
 
 
-DWORD RPC_MainLoop(void)
+DWORD RPC_Init(void)
 {
     WCHAR transport[] = SVCCTL_TRANSPORT;
     WCHAR endpoint[] = SVCCTL_ENDPOINT;
-    HANDLE hSleepHandle;
     DWORD err;
 
     if ((err = RpcServerUseProtseqEpW(transport, 0, endpoint, NULL)) != ERROR_SUCCESS)
@@ -1348,7 +1129,7 @@ DWORD RPC_MainLoop(void)
 
     if ((err = RpcServerRegisterIf(svcctl_v2_0_s_ifspec, 0, 0)) != ERROR_SUCCESS)
     {
-        WINE_ERR("RpcServerRegisterIf failed with error %u", err);
+        WINE_ERR("RpcServerRegisterIf failed with error %u\n", err);
         return err;
     }
 
@@ -1357,17 +1138,26 @@ DWORD RPC_MainLoop(void)
         WINE_ERR("RpcServerListen failed with error %u\n", err);
         return err;
     }
+    return ERROR_SUCCESS;
+}
+
+DWORD RPC_MainLoop(void)
+{
+    DWORD err;
+    HANDLE hExitEvent = __wine_make_process_system();
+
+    SetEvent(g_hStartedEvent);
 
     WINE_TRACE("Entered main loop\n");
-    hSleepHandle = __wine_make_process_system();
-    SetEvent(g_hStartedEvent);
+
     do
     {
-        err = WaitForSingleObjectEx(hSleepHandle, INFINITE, TRUE);
+        err = WaitForSingleObjectEx(hExitEvent, INFINITE, TRUE);
         WINE_TRACE("Wait returned %d\n", err);
     } while (err != WAIT_OBJECT_0);
 
     WINE_TRACE("Object signaled - wine shutdown\n");
+    CloseHandle(hExitEvent);
     return ERROR_SUCCESS;
 }
 
