@@ -53,7 +53,9 @@
 
 static struct list process_list = LIST_INIT(process_list);
 static int running_processes, user_processes;
-static struct event *user_process_event;  /* signaled when all user processes have exited */
+static struct event *shutdown_event;           /* signaled when shutdown starts */
+static struct timeout_user *shutdown_timeout;  /* timeout for server shutdown */
+static int shutdown_stage;  /* current stage in the shutdown process */
 
 /* process operations */
 
@@ -67,6 +69,7 @@ static const struct object_ops process_ops =
 {
     sizeof(struct process),      /* size */
     process_dump,                /* dump */
+    no_get_type,                 /* get_type */
     add_queue,                   /* add_queue */
     remove_queue,                /* remove_queue */
     process_signaled,            /* signaled */
@@ -74,6 +77,8 @@ static const struct object_ops process_ops =
     no_signal,                   /* signal */
     no_get_fd,                   /* get_fd */
     process_map_access,          /* map_access */
+    default_get_sd,              /* get_sd */
+    default_set_sd,              /* set_sd */
     no_lookup_name,              /* lookup_name */
     no_open_file,                /* open_file */
     no_close_handle,             /* close_handle */
@@ -114,6 +119,7 @@ static const struct object_ops startup_info_ops =
 {
     sizeof(struct startup_info),   /* size */
     startup_info_dump,             /* dump */
+    no_get_type,                   /* get_type */
     add_queue,                     /* add_queue */
     remove_queue,                  /* remove_queue */
     startup_info_signaled,         /* signaled */
@@ -121,6 +127,8 @@ static const struct object_ops startup_info_ops =
     no_signal,                     /* signal */
     no_get_fd,                     /* get_fd */
     no_map_access,                 /* map_access */
+    default_get_sd,                /* get_sd */
+    default_set_sd,                /* set_sd */
     no_lookup_name,                /* lookup_name */
     no_open_file,                  /* open_file */
     no_close_handle,               /* close_handle */
@@ -140,6 +148,8 @@ static unsigned int alloc_ptid_entries;     /* number of allocated entries */
 static unsigned int next_free_ptid;         /* next free entry */
 static unsigned int last_free_ptid;         /* last free entry */
 
+static void kill_all_processes(void);
+
 #define PTID_OFFSET 8  /* offset for first ptid value */
 
 /* allocate a new process or thread id */
@@ -221,17 +231,53 @@ static void set_process_startup_state( struct process *process, enum startup_sta
     }
 }
 
+/* callback for server shutdown */
+static void server_shutdown_timeout( void *arg )
+{
+    shutdown_timeout = NULL;
+    if (!running_processes)
+    {
+        close_master_socket( 0 );
+        return;
+    }
+    switch(++shutdown_stage)
+    {
+    case 1:  /* signal system processes to exit */
+        if (debug_level) fprintf( stderr, "wineserver: shutting down\n" );
+        if (shutdown_event) set_event( shutdown_event );
+        shutdown_timeout = add_timeout_user( 2 * -TICKS_PER_SEC, server_shutdown_timeout, NULL );
+        close_master_socket( 4 * -TICKS_PER_SEC );
+        break;
+    case 2:  /* now forcibly kill all processes (but still wait for SIGKILL timeouts) */
+        kill_all_processes();
+        break;
+    }
+}
+
+/* forced shutdown, used for wineserver -k */
+void shutdown_master_socket(void)
+{
+    kill_all_processes();
+    shutdown_stage = 2;
+    if (shutdown_timeout)
+    {
+        remove_timeout_user( shutdown_timeout );
+        shutdown_timeout = NULL;
+    }
+    close_master_socket( 2 * -TICKS_PER_SEC );  /* for SIGKILL timeouts */
+}
+
 /* final cleanup once we are sure a process is really dead */
 static void process_died( struct process *process )
 {
     if (debug_level) fprintf( stderr, "%04x: *process killed*\n", process->id );
     if (!process->is_system)
     {
-        if (!--user_processes && user_process_event)
-            set_event( user_process_event );
+        if (!--user_processes && !shutdown_stage && master_socket_timeout != TIMEOUT_INFINITE)
+            shutdown_timeout = add_timeout_user( master_socket_timeout, server_shutdown_timeout, NULL );
     }
     release_object( process );
-    if (!--running_processes) close_master_socket();
+    if (!--running_processes && shutdown_stage) close_master_socket( 0 );
 }
 
 /* callback for process sigkill timeout */
@@ -276,7 +322,7 @@ struct thread *create_process( int fd, struct thread *parent_thread, int inherit
     process->exit_code       = STILL_ACTIVE;
     process->running_threads = 0;
     process->priority        = PROCESS_PRIOCLASS_NORMAL;
-    process->affinity        = 1;
+    process->affinity        = ~0;
     process->suspend         = 0;
     process->is_system       = 0;
     process->create_flags    = 0;
@@ -298,7 +344,7 @@ struct thread *create_process( int fd, struct thread *parent_thread, int inherit
 
     process->start_time = current_time;
     process->end_time = 0;
-    list_add_head( &process_list, &process->entry );
+    list_add_tail( &process_list, &process->entry );
 
     if (!(process->id = process->group_id = alloc_ptid( process )))
     {
@@ -347,7 +393,7 @@ struct thread *create_process( int fd, struct thread *parent_thread, int inherit
  error:
     if (process) release_object( process );
     /* if we failed to start our first process, close everything down */
-    if (!running_processes) close_master_socket();
+    if (!running_processes) close_master_socket( 0 );
     return NULL;
 }
 
@@ -413,18 +459,7 @@ static void process_poll_event( struct fd *fd, int event )
     struct process *process = get_fd_user( fd );
     assert( process->obj.ops == &process_ops );
 
-    if (event & (POLLERR | POLLHUP))
-    {
-        release_object( process->msg_fd );
-        process->msg_fd = NULL;
-        if (process->sigkill_timeout)  /* already waiting for it to die */
-        {
-            remove_timeout_user( process->sigkill_timeout );
-            process->sigkill_timeout = NULL;
-            process_died( process );
-        }
-        else kill_process( process, 0 );
-    }
+    if (event & (POLLERR | POLLHUP)) kill_process( process, 0 );
     else if (event & POLLIN) receive_fd( process );
 }
 
@@ -552,7 +587,7 @@ static void terminate_process( struct process *process, struct thread *skip, int
 }
 
 /* kill all processes */
-void kill_all_processes( struct process *skip, int exit_code )
+static void kill_all_processes(void)
 {
     for (;;)
     {
@@ -560,11 +595,10 @@ void kill_all_processes( struct process *skip, int exit_code )
 
         LIST_FOR_EACH_ENTRY( process, &process_list, struct process, entry )
         {
-            if (process == skip) continue;
             if (process->running_threads) break;
         }
         if (&process->entry == &process_list) break;  /* no process found */
-        terminate_process( process, NULL, exit_code );
+        terminate_process( process, NULL, 1 );
     }
 }
 
@@ -595,7 +629,7 @@ static void process_killed( struct process *process )
 
     assert( list_empty( &process->thread_list ));
     process->end_time = current_time;
-    close_process_desktop( process );
+    if (!process->is_system) close_process_desktop( process );
     handles = process->handles;
     process->handles = NULL;
     if (handles) release_object( handles );
@@ -628,8 +662,11 @@ void add_process_thread( struct process *process, struct thread *thread )
         running_processes++;
         if (!process->is_system)
         {
-            if (!user_processes++ && user_process_event)
-                reset_event( user_process_event );
+            if (!user_processes++ && shutdown_timeout)
+            {
+                remove_timeout_user( shutdown_timeout );
+                shutdown_timeout = NULL;
+            }
         }
     }
     grab_object( thread );
@@ -688,6 +725,20 @@ void resume_process( struct process *process )
 /* kill a process on the spot */
 void kill_process( struct process *process, int violent_death )
 {
+    if (!violent_death && process->msg_fd)  /* normal termination on pipe close */
+    {
+        release_object( process->msg_fd );
+        process->msg_fd = NULL;
+    }
+
+    if (process->sigkill_timeout)  /* already waiting for it to die */
+    {
+        remove_timeout_user( process->sigkill_timeout );
+        process->sigkill_timeout = NULL;
+        process_died( process );
+        return;
+    }
+
     if (violent_death) terminate_process( process, NULL, 1 );
     else
     {
@@ -854,6 +905,12 @@ DECL_HANDLER(new_process)
         close( socket_fd );
         return;
     }
+    if (shutdown_stage)
+    {
+        set_error( STATUS_SHUTDOWN_IN_PROGRESS );
+        close( socket_fd );
+        return;
+    }
 
     /* build the startup info for a new process */
     if (!(info = alloc_object( &startup_info_ops ))) return;
@@ -987,7 +1044,7 @@ DECL_HANDLER(init_process_done)
     generate_startup_debug_events( process, req->entry );
     set_process_startup_state( process, STARTUP_DONE );
 
-    if (req->gui) process->idle_event = create_event( NULL, NULL, 0, 1, 0 );
+    if (req->gui) process->idle_event = create_event( NULL, NULL, 0, 1, 0, NULL );
     if (current->suspend + process->suspend > 0) stop_thread( current );
     if (process->debugger) set_process_debug_flag( process, 1 );
 }
@@ -1044,11 +1101,7 @@ DECL_HANDLER(set_process_info)
     if ((process = get_process_from_handle( req->handle, PROCESS_SET_INFORMATION )))
     {
         if (req->mask & SET_PROCESS_INFO_PRIORITY) process->priority = req->priority;
-        if (req->mask & SET_PROCESS_INFO_AFFINITY)
-        {
-            if (req->affinity != 1) set_error( STATUS_INVALID_PARAMETER );
-            else process->affinity = req->affinity;
-        }
+        if (req->mask & SET_PROCESS_INFO_AFFINITY) process->affinity = req->affinity;
         release_object( process );
     }
 }
@@ -1125,16 +1178,25 @@ DECL_HANDLER(get_dll_info)
 
     if ((process = get_process_from_handle( req->handle, PROCESS_QUERY_INFORMATION )))
     {
-        struct process_dll *dll = find_process_dll( process, req->base_address );
+        struct process_dll *dll;
+
+        if (req->base_address)
+            dll = find_process_dll( process, req->base_address );
+        else /* NULL means main module */
+            dll = list_head( &process->dlls ) ?
+                LIST_ENTRY(list_head( &process->dlls ), struct process_dll, entry) : NULL;
 
         if (dll)
         {
             reply->size = dll->size;
             reply->entry_point = NULL; /* FIXME */
+            reply->filename_len = dll->namelen;
             if (dll->filename)
             {
-                data_size_t len = min( dll->namelen, get_reply_max_size() );
-                set_reply_data( dll->filename, len );
+                if (dll->namelen <= get_reply_max_size())
+                    set_reply_data( dll->filename, dll->namelen );
+                else
+                    set_error( STATUS_BUFFER_TOO_SMALL );
             }
         }
         else
@@ -1164,18 +1226,20 @@ DECL_HANDLER(make_process_system)
 {
     struct process *process = current->process;
 
-    if (!user_process_event)
+    if (!shutdown_event)
     {
-        if (!(user_process_event = create_event( NULL, NULL, 0, 1, 0 ))) return;
-        make_object_static( (struct object *)user_process_event );
+        if (!(shutdown_event = create_event( NULL, NULL, 0, 1, 0, NULL ))) return;
+        make_object_static( (struct object *)shutdown_event );
     }
 
-    if (!(reply->event = alloc_handle( current->process, user_process_event, EVENT_ALL_ACCESS, 0 )))
+    if (!(reply->event = alloc_handle( current->process, shutdown_event, SYNCHRONIZE, 0 )))
         return;
 
     if (!process->is_system)
     {
         process->is_system = 1;
-        if (!--user_processes) set_event( user_process_event );
+        close_process_desktop( process );
+        if (!--user_processes && !shutdown_stage && master_socket_timeout != TIMEOUT_INFINITE)
+            shutdown_timeout = add_timeout_user( master_socket_timeout, server_shutdown_timeout, NULL );
     }
 }
